package org.processmining.plugins.complianceframework;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeSet;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ImportanceConnection;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.DataElement;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PNWDTransition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PetriNetWithData;
import org.processmining.models.guards.Expression;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.DataConformance.GUI.MappingPanel;
import org.processmining.plugins.DataConformance.framework.ExecutionStep;
import org.processmining.plugins.compliance.AbstractLogForCompliance_Plugin;
import org.processmining.plugins.compliance.align.PNLogReplayer;
import org.processmining.plugins.compliance.align.PNLogReplayer.ReplayParams;
import org.processmining.plugins.compliance.temporal.CreateTemporalPattern_Plugin;
import org.processmining.plugins.complianceframework.model.ComplianceDashboard;
import org.processmining.plugins.complianceframework.model.DashboardAlignment;
import org.processmining.plugins.complianceframework.model.DashboardResultReplay;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.datacompliance.DataComplianceHelper;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;

public class DeviationOverviewPlugin {
	protected static int maxIntervalSize = 120000;

	@Plugin(name = "Get Compliance dashboard", parameterLabels = { "Log", "Data Petri Net" }, returnLabels = { "Compliance Dashboard" }, returnTypes = { ComplianceDashboard.class })
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "V. Gromov", email = "v.gromov@student.tue.nl")
	public ComplianceDashboard getDeviationOverview(UIPluginContext context, XLog log, PetriNetWithData DPNet) {
		// TODO: detect/ask resource variable
		ImportanceConnection importanceConnection = connectPNToImportanceMap(DPNet, context);
		Map<String, Float> activityImportanceMap = importanceConnection
				.getObjectWithRole(ImportanceConnection.ACTVITYIMPORTANCEMAPPING);
		Map<String, Float> attributeImportanceMap = importanceConnection
				.getObjectWithRole(ImportanceConnection.ATTRIBUTEIMPORTANCEMAPPING);

		XEventClassifier[] availableEventCLasses = new XEventClassifier[4];
		availableEventCLasses[0] = XLogInfoImpl.STANDARD_CLASSIFIER;
		availableEventCLasses[1] = XLogInfoImpl.NAME_CLASSIFIER;
		availableEventCLasses[2] = XLogInfoImpl.RESOURCE_CLASSIFIER;
		availableEventCLasses[3] = XLogInfoImpl.LIFECYCLE_TRANSITION_CLASSIFIER;
		Object[] mappingOutput = DataComplianceHelper.getActivitiesMapping(context, log, DPNet, availableEventCLasses);
		if (mappingOutput == null) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		TransEvClassMapping mapping = (TransEvClassMapping) mappingOutput[0];
		XEventClassifier selectedClassifier = (XEventClassifier) mappingOutput[1];
		XLog abstractedLog = DataComplianceHelper.abstractLog(log, DPNet, mapping, selectedClassifier);

		PNRepResult res = DataComplianceHelper.cfReplayLog(context, DPNet, mapping, abstractedLog);
		if (res == null) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		//		PNRepResult res = (PNRepResult) replayOutput[0];
		//		TransEvClassMapping mapping = (TransEvClassMapping) replayOutput[1];
		//		XLog abstractedLog = (XLog) replayOutput[2];
		XLogInfo summary = XLogInfoFactory.createLogInfo(abstractedLog, selectedClassifier);

		Collection<XEventClass> unmappedClasses = summary.getEventClasses().getClasses();

		for (XEventClass evClass : mapping.values()) {
			unmappedClasses.remove(evClass);
		}
		for (XEventClass unmapped_EvClass : unmappedClasses) {
			if (!activityImportanceMap.containsKey(unmapped_EvClass.toString()))
				activityImportanceMap.put(unmapped_EvClass.toString(), 0.5f);
		}

		// register them in the framework
		context.getProvidedObjectManager().createProvidedObject(
				"Control-flow compliance alignment for " + XConceptExtension.instance().extractName(abstractedLog)
						+ " on " + DPNet.getLabel(), res, PNRepResult.class, context);
		context.getProvidedObjectManager().createProvidedObject(
				"Mapping between " + XConceptExtension.instance().extractName(abstractedLog) + " and "
						+ DPNet.getLabel(), mapping, TransEvClassMapping.class, context);
		//		DataAlignment dataAligner = new DataAlignment();

		//Variable Mapping
		Collection<String> logAttributes = summary.getEventAttributeInfo().getAttributeKeys();
		for (int i = 0; i < DPNReplayer.metaAttributes.length; i++) {
			if (logAttributes.contains(DPNReplayer.metaAttributes[i])) {
				logAttributes.remove(DPNReplayer.metaAttributes[i]);
			}
		}

		TreeSet<String> processAttributes = new TreeSet<String>();
		for (DataElement variable : DPNet.getVariables()) {
			processAttributes.add(variable.getVarName());
		}
		MappingPanel<String, String> mapVariablePanel = new MappingPanel<String, String>(processAttributes,
				logAttributes);

		InteractionResult interactionResult = context.showConfiguration("Setup Variable Mapping", mapVariablePanel);
		if (interactionResult == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		Map<String, String> variableMapping = mapVariablePanel.getMapping(false);
		//Guard mappring
		Map<String, Expression> guardsMap = DataComplianceHelper.getGuardsMap(DPNet);

		Map<String, Transition> transitionsMap = DataComplianceHelper.getTransitionsMap(DPNet);

		Map<String, Class<?>> variableTypes = new HashMap<>();
		for (DataElement variable: DPNet.getVariables()) {
			variableTypes.put(variable.getVarName(), variable.getType());
		}
		DashboardResultReplay repResWithData = DPNReplayer.replayDPN(context, abstractedLog, res, variableMapping,
				guardsMap, transitionsMap, variableTypes, false);
		//		ResultReplayPetriNetWithData repResWithData = dataAligner.plugin(context, DPNet, abstractedLog);

		//		context.getProvidedObjectManager().createProvidedObject(repResWithData.toString(), repResWithData,
		//				ResultReplayPetriNetWithData.class, context);

		Map<String, String> evClassActivityMap = mapEvClassToTransition(mapping);

		Map<String, float[]> intervals = getIntervals(DPNet, repResWithData);

		DashboardResultReplay repResTemporal = TemporalReplayer.checkTemporalCompliance(context, res, abstractedLog,
				evClassActivityMap);

		ComplianceDashboard result;
		//		if (context.showConfiguration("Temporal Compliance", new TemporalDecisionPanel()) == InteractionResult.CANCEL) {
		result = new ComplianceDashboard(repResWithData, repResTemporal, intervals, activityImportanceMap,
				attributeImportanceMap, evClassActivityMap, abstractedLog, log);
		//		} else {
		//					org.processmining.plugins.compliance.temporal.CheckComplianceReplayer_Plugin temporalCompliancePlugin = new org.processmining.plugins.compliance.temporal.CheckComplianceReplayer_Plugin();
		//					ResultReplay repResTemporal = (ResultReplay) temporalCompliancePlugin.replayLog(context, res)[0];
		//					context.getProvidedObjectManager().createProvidedObject(repResTemporal.toString(), repResTemporal,
		//							ResultReplay.class, context);
		//					result = new ComplianceDashboard(res, repResWithData, repResTemporal, false, null, intervals,
		//							activityImportanceMap, attributeImportanceMap, evClassActivityMap, abstractedLog, context);
		//		}

		return result;
		//		return null;
	}

	//	public XLog enrichLog(UIPluginContext context, XLog abstractedLog, TemporalComplianceRequirement req) {
	//		XFactory f = XFactoryRegistry.instance().currentDefault();
	//
	//		XLog logNew = f.createLog(abstractedLog.getAttributes());
	//		for (XTrace trace : abstractedLog) {
	//			XTrace traceNew = f.createTrace(trace.getAttributes());
	//
	//			// give each event that has a valid time stamp a second time-stamp attribute prefixed by its name
	//			for (int i = 0; i < trace.size(); i++) {
	//				XEvent e = trace.get(i);
	//				XEvent eNew = f.createEvent(e.getAttributes());
	//
	//				Date timeStamp = XTimeExtension.instance().extractTimestamp(eNew);
	//
	//				if (timeStamp != null) {
	//					XEventClass eClass = req.allEventClasses.getClassOf(eNew);
	//					String timeStampName = getEventSpecificTimestampName(eClass);
	//					setEventSpecificTimestamp(eNew, timeStampName, timeStamp, f);
	//				}
	//				traceNew.add(eNew);
	//			}
	//
	//			logNew.add(traceNew);
	//		}
	//
	//		DateFormat dateFormat = new SimpleDateFormat("HH:mm:ss");
	//		Date date = new Date();
	//		String timeString = dateFormat.format(date);
	//
	//		return logNew;
	//	}

	private void setEventNameAndLifeCycle(XEvent e, Object aligned, StepTypes type, TransEvClassMapping map, XFactory f) {
		// split name into event name and life-cycle transition
		String qualified_eventName;

		qualified_eventName = aligned.toString();

		String name;
		String life_cycle;
		int plus_pos = qualified_eventName.indexOf('+');
		if (plus_pos >= 0) {
			name = qualified_eventName.substring(0, plus_pos);
			life_cycle = qualified_eventName.substring(plus_pos + 1);
		} else {
			name = qualified_eventName;
			life_cycle = "complete";
		}

		XConceptExtension.instance().assignName(e, name);
		XLifecycleExtension.instance().assignTransition(e, life_cycle);
	}

	protected Map<String, String> mapEvClassToTransition(TransEvClassMapping mapping) {
		Map<String, String> result = new HashMap<String, String>();

		for (Entry<Transition, XEventClass> mapEntry : mapping.entrySet()) {
			String evClass = mapEntry.getValue().toString();
			List<String> keysList = new ArrayList<String>();
			for (Entry<Transition, XEventClass> innerMapEntry : mapping.entrySet()) {
				if (innerMapEntry.getValue().toString().equals(evClass))
					keysList.add(innerMapEntry.getKey().getLabel());
			}
			String[] keys = keysList.toArray(new String[1]);
			Arrays.sort(keys);
			//			String evClassNoTr = null;
			//			if (evClass.endsWith("+complete"))
			//				evClassNoTr = evClass.replace("+complete", "");
			//			else if (evClass.endsWith("+start"))
			//				evClassNoTr = evClass.replace("+start", "");
			//			else if (evClass.endsWith("+"))
			//				evClassNoTr = evClass.replace("+", "");
			//			String activityName = mapEntry.getKey().getLabel();
			String activityName = keys[0];
			if (!result.containsKey(evClass))
				result.put(evClass, activityName);
			//			if (evClassNoTr != null && !result.containsKey(evClassNoTr))
			//				result.put(evClassNoTr, activityName);
			if (evClass != null && !result.containsKey(evClass))
				result.put(evClass, activityName);
		}
		return result;
	}

	// get control-flow alignment in almost the same way, as in compliance
	// checking
	protected Object[] replayLog(UIPluginContext context, PetrinetGraph net, XLog originalLog) {
		for (Transition t : net.getTransitions()) {
			if (t.getLabel().toLowerCase().equalsIgnoreCase("tau")
					|| t.getLabel().toLowerCase().startsWith("final")
					|| t.getLabel().toLowerCase().equals(CreateTemporalPattern_Plugin.T_INSTANCE_START.toLowerCase())
					|| t.getLabel().toLowerCase()
							.equals(CreateTemporalPattern_Plugin.T_INSTANCE_COMPLETE.toLowerCase())
					|| t.getLabel().toLowerCase().equals(CreateTemporalPattern_Plugin.T_START.toLowerCase())
					|| t.getLabel().toLowerCase().equals(CreateTemporalPattern_Plugin.T_END.toLowerCase())) {
				t.setInvisible(true);
			}
		}

		Object[] abstraction = AbstractLogForCompliance_Plugin.connectLogToPetriNet(context, originalLog, net);
		if (abstraction == null) {
			return null;
		}

		XLog log = (XLog) abstraction[0];

		TransEvClassMapping mapping = (TransEvClassMapping) abstraction[1];

		Marking m_initial;
		try {
			m_initial = context.tryToFindOrConstructFirstObject(Marking.class, InitialMarkingConnection.class,
					InitialMarkingConnection.MARKING, net);
		} catch (ConnectionCannotBeObtained e) {
			return null;
		}

		Marking m_final = (Marking) PNLogReplayer.constructFinalMarking(context, net)[1];

		ReplayParams par = PNLogReplayer.getReplayerParameters(context, net, log, mapping.getEventClassifier(),
				m_initial, m_final);
		for (XEventClass e : par.mapEvClass2Cost.keySet()) {
			par.mapEvClass2Cost.put(e, 100);
		}
		par.parameters.setCreateConn(true);
		PNRepResult res = PNLogReplayer.callReplayer(context, net, log, mapping, par);

		context.getFutureResult(0).setLabel(
				"Control-flow compliance alignment for " + XConceptExtension.instance().extractName(log) + " on "
						+ net.getLabel() + " using " + par.selectedAlg.toString());

		return new Object[] { res, mapping, log };
	}

	protected ImportanceConnection connectPNToImportanceMap(PetrinetGraph net, UIPluginContext context) {
		Map<String, Float> activityImportanceMap = null;
		Map<String, Float> attributeImportanceMap = null;
		ImportanceConnection connection = null;
		try {
			connection = context.getConnectionManager().getFirstConnection(ImportanceConnection.class, context, net);
			activityImportanceMap = connection.getObjectWithRole(ImportanceConnection.ACTVITYIMPORTANCEMAPPING);
			attributeImportanceMap = connection.getObjectWithRole(ImportanceConnection.ATTRIBUTEIMPORTANCEMAPPING);
		} catch (ConnectionCannotBeObtained e1) {
			try {
				activityImportanceMap = new HashMap<String, Float>();
				attributeImportanceMap = new HashMap<String, Float>();
				if (net instanceof PetriNetWithData)
					for (DataElement var : ((PetriNetWithData) net).getVariables()) {
						attributeImportanceMap.put(var.getVarName(), 0.5f);
					}
				for (Transition activity : net.getTransitions()) {
					activityImportanceMap.put(activity.getLabel(), 0.5f);
					//					if (((PNWDTransition) activity).getGuard() != null) {
					//						Enumeration vars = ((PNWDTransition) activity).getGuard().findVariables();
					//						while (vars.hasMoreElements())
					//							attributeImportanceMap.put(((String) vars.nextElement()).replace("'", ""), 1.0f);
					//					}
				}

				connection = new ImportanceConnection("Connection between " + net.getLabel() + " and importance", net,
						activityImportanceMap, attributeImportanceMap);
				context.getConnectionManager().addConnection(connection);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return connection;
	}

	protected Map<String, float[]> getIntervals(PetriNetWithData DPNet, DashboardResultReplay dataAwareReplay) {
		Map<String, float[]> result = new HashMap<String, float[]>();
		for (DataElement variable : DPNet.getVariables()) {
			if (!variable.getType().equals(Float.class) && !variable.getType().equals(Integer.class))
				continue;
			List<Float> valuesList = new ArrayList<Float>();
			boolean intervalsInDPN = true;
			for (Transition t : DPNet.getTransitions()) {
				Expression expression = ((PNWDTransition) t).getGuard();
				if (expression == null)
					continue;
				if (!ExpressionHelper.extractGuardNumbers(expression, variable, valuesList)) {
					intervalsInDPN = false;
					break;
				}
			}
			if (!intervalsInDPN || valuesList.size() == 0)
				result.put(variable.getVarName(), getIntervalsFromLog(variable, dataAwareReplay));
			else {
				valuesList.add(Float.MAX_VALUE);
				for (int i = 0; i < valuesList.size() - 1; i++) {
					for (int j = i + 1; j < valuesList.size(); j++) {
						if (valuesList.get(i).floatValue() == valuesList.get(j).floatValue()) {
							valuesList.remove(j);
							j -= 1;
						}
					}
				}
				float[] values = new float[valuesList.size()];
				for (int i = 0; i < values.length; i++) {
					values[i] = valuesList.get(i).floatValue();
				}
				Arrays.sort(values);
				result.put(variable.getVarName(), values);
			}
		}
		return result;
	}

	protected float[] getIntervalsFromLog(DataElement variable, DashboardResultReplay dataAwareReplay) {
		float min = Float.MAX_VALUE, max = -Float.MAX_VALUE;
		for (DashboardAlignment alignment : dataAwareReplay.labelStepArray) {
			for (ExecutionStep LogStep : alignment.getLogTrace()) {
				Object varVal = LogStep.get(variable.getVarName());
				if (varVal == null)
					continue;
				float value = Float.parseFloat(varVal.toString());
				if (value > max) {
					max = value;
				}
				if (value < min) {
					min = value;
				}
			}
		}
		if (max == -Float.MAX_VALUE && min == Float.MAX_VALUE)
			return null;
		if (min == max) {
			return new float[] { min, Float.MAX_VALUE };
		}

		int numIntervals = 10;
		float intervalSize = (max - min) / numIntervals;
		if ((int) intervalSize > maxIntervalSize) {
			intervalSize = maxIntervalSize;
			numIntervals = (int) ((max - min) / intervalSize);
			if (numIntervals * intervalSize < max) {
				numIntervals += 1;
			}
		}
		float[] result = new float[numIntervals + 2];
		for (int i = 0; i < result.length - 1; i++) {
			result[i] = min + intervalSize * (float) i;
		}
		result[result.length - 1] = Float.MAX_VALUE;

		return result;
	}
}