package org.processmining.plugins.complianceframework.GUI;

import java.awt.FontMetrics;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import org.processmining.plugins.complianceframework.model.ComplianceDashboard;

public class AttributeImportanceTable extends JTable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public AttributeImportanceTable(ComplianceDashboard dashboard, DashboardJTree tree) {
		super(new AttributeImportanceTableModel(dashboard, tree));
	}

	@Override
	public boolean isCellEditable(int row, int col) {
		switch (col) {
			case 0 :
				return false;
			case 1 :
				return true;
			default :
				return false;
		}
	}
	
	@Override
	public void setFont(java.awt.Font font) {
		super.setFont(font);
		this.getTableHeader().setFont(font);
		FontMetrics fontMetrics = this.getFontMetrics(font);
		this.setRowHeight(fontMetrics.getHeight());
	}
}

class AttributeImportanceTableModel extends DefaultTableModel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	Map<String, Float> attributeImportanceMap;
	ComplianceDashboard dashboard;
	DashboardJTree tree;

	public AttributeImportanceTableModel(ComplianceDashboard dashboard, DashboardJTree tree) {
		super();
		this.attributeImportanceMap = dashboard.getAttributeImportanceMap();
		this.dashboard = dashboard;
		this.tree = tree;
		init();
	}

	private void init() {

		Object[] headers = new Object[] { "Attribute", "Importance (between 0.0 and 1.0)" };
		Object[][] values = new Object[this.attributeImportanceMap.entrySet().size()][];
		int i = 0;
		for (Entry<String, Float> activityImportanceEntry : this.attributeImportanceMap.entrySet()) {
			values[i] = new Object[2];
			values[i][0] = activityImportanceEntry.getKey();
			values[i][1] = activityImportanceEntry.getValue().floatValue();
			i++;
		}

		boolean sorted;
		do {
			sorted = true;
			for (int j = 0; j < values.length - 1; j++) {
				if (values[j][0].toString().compareTo(values[j + 1][0].toString()) > 0) {
					sorted = false;
					Object[] buf = values[j + 1];
					values[j + 1] = values[j];
					values[j] = buf;
				}
			}
		} while (!sorted);

		setDataVector(values, headers);

		addTableModelListener(new TableModelListener() {

			public void tableChanged(TableModelEvent e) {
				int row = e.getFirstRow();
				TableModel model = (TableModel) e.getSource();
				String newValueString = model.getValueAt(row, 1).toString();
				float newValue;
				try {
					newValue = Float.parseFloat(newValueString);
					if ((newValue <= 1.0f) && (newValue >= 0.0f)) {
						AttributeImportanceTableModel.this.attributeImportanceMap.put(
								model.getValueAt(row, 0).toString(), newValue);
//						AttributeImportanceTableModel.this.dashboard.generateStatements();
						GUIUtilities.reInitJTree(AttributeImportanceTableModel.this.tree, AttributeImportanceTableModel.this.dashboard);
						//						AttributeImportanceTableModel.this.tree.init(
						//								AttributeImportanceTableModel.this.dashboard.getStatementTreeModel(),
						//								AttributeImportanceTableModel.this.dashboard.nodeProblemMap);
					} else {
						model.setValueAt(AttributeImportanceTableModel.this.attributeImportanceMap.get(model.getValueAt(
								row, 0).toString()), row, 1);
					}
				} catch (Exception ex) {
					model.setValueAt(AttributeImportanceTableModel.this.attributeImportanceMap.get(model.getValueAt(row,
							0).toString()), row, 1);
				}
			}
		});
	}
}