package org.processmining.plugins.complianceframework.GUI;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultHighlighter;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeSelectionModel;

import org.processmining.plugins.complianceframework.model.ProblemSentenceObject;

public class DashboardJTree extends JTree {
	private static final String[] CSV_HEADERS = new String[]{"Violation", "Attribute", "value", "#Violations", "CPIR", "Value Implies violation"};

	private static final long serialVersionUID = 1L;
	private static final String lineSeparator = System.getProperty("line.separator");
	private boolean allowTopSelection;

	public Map<DefaultMutableTreeNode, ProblemSentenceObject> nodeProblemMap;

	protected List<TreeModelUpdatedListener> modelUpdatedListeners;

	public DashboardJTree(boolean allowTopSelection) {
		this(null, null, allowTopSelection);
	}

	public DashboardJTree(DefaultTreeModel treeModel,
			Map<DefaultMutableTreeNode, ProblemSentenceObject> nodeProblemMap, boolean allowTopSelection) {
		super();
		modelUpdatedListeners = new ArrayList<TreeModelUpdatedListener>();
		this.allowTopSelection = allowTopSelection;
		this.setTreeHandles();
		if (treeModel != null && nodeProblemMap != null)
			init(treeModel, nodeProblemMap, allowTopSelection);
	}

	public void init(DefaultTreeModel treeModel, Map<DefaultMutableTreeNode, ProblemSentenceObject> nodeProblemMap) {
		this.init(treeModel, nodeProblemMap, this.allowTopSelection);
	}

	public void init(DefaultTreeModel treeModel, Map<DefaultMutableTreeNode, ProblemSentenceObject> nodeProblemMap,
			boolean allowTopSelection) {
		//		try {
		////			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		//			UIManager.setLookAndFeel("com.sun.java.swing.plaf.gtk.GTKLookAndFeel");
		//		} catch (Exception e) {
		//			// TODO Auto-generated catch block
		//			e.printStackTrace();
		//		}
		this.setModel(treeModel);
		this.nodeProblemMap = nodeProblemMap;
		this.setRowHeight(0);
		DasboardTreeCellRenderer cellRenderer = new DasboardTreeCellRenderer(nodeProblemMap, allowTopSelection);
		this.setCellRenderer(cellRenderer);
		this.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		this.setRootVisible(false);
		this.setShowsRootHandles(true);
		if (!allowTopSelection)
			this.setSelectionListener(treeModel);
		ToolTipManager.sharedInstance().registerComponent(this);
	}

	@Override
	public void setModel(TreeModel newModel) {
		super.setModel(newModel);
		this.treeUpdated();
	}
	
	public void treeUpdated(){
		if (this.modelUpdatedListeners == null || this.modelUpdatedListeners.size() == 0)
			return;
		for (TreeModelUpdatedListener listener : this.modelUpdatedListeners)
			listener.modelChanged();
	}

	public void setTreeHandles() {
		try {
			Thread thread = Thread.currentThread();
			ClassLoader classLoader = thread.getContextClassLoader();
			InputStream input = classLoader.getResourceAsStream("images/expanded.png");
			Icon expanded = new ImageIcon(ImageIO.read(input));
			input.close();
			input = classLoader.getResourceAsStream("images/collapsed.png");
			Icon collapsed = new ImageIcon(ImageIO.read(input));
			input.close();
			UIManager.put("Tree.collapsedIcon", collapsed);
			UIManager.put("Tree.expandedIcon", expanded);
			SwingUtilities.updateComponentTreeUI(this);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	//	private ComponentListener componentListener = new ComponentListener() {
	//		
	//		public void componentShown(ComponentEvent e) {
	//			// TODO Auto-generated method stub
	//			
	//		}
	//		
	//		public void componentResized(ComponentEvent e) {
	//			if (this.			
	//		}
	//		
	//		public void componentMoved(ComponentEvent e) {
	//			// TODO Auto-generated method stub
	//			
	//		}
	//		
	//		public void componentHidden(ComponentEvent e) {
	//			// TODO Auto-generated method stub
	//			
	//		}
	//	};
	//	
	//	private int getNodeWidth(TreeNode node){
	//	}

	private void setSelectionListener(final DefaultTreeModel treeModel) {
		this.addTreeSelectionListener(new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent e) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) DashboardJTree.this
						.getLastSelectedPathComponent();
				if (treeModel.getIndexOfChild(treeModel.getRoot(), node) != -1)
					DashboardJTree.this.clearSelection();
			}
		});
	}

	public boolean getAllowTopSelection() {
		return this.allowTopSelection;
	}

	public void addModelUpdatedListener(TreeModelUpdatedListener listener) {
		this.modelUpdatedListeners.add(listener);
	}

	public void removeModelUpdatedListener(int index) {
		this.modelUpdatedListeners.remove(index);
	}

	public void removeModelUpdatedListener(TreeModelUpdatedListener listener) {
		this.modelUpdatedListeners.remove(listener);
	}

	public void removeModelUpdatedListener() {
		this.modelUpdatedListeners.remove(this.modelUpdatedListeners.size() - 1);
	}

	public void removeAllModelUpdatedListeners() {
		this.modelUpdatedListeners.clear();
	}
	
	@Override
	public String toString() {
		StringWriter writer = new StringWriter();
		try {
			this.writeToWriter(writer);
		}
		catch (IOException ex) {
			System.err.println("Error while converting tree to string");
			return null;
		}
		return writer.toString();
	}
	
	public void writeToWriter(Writer writer) throws IOException {
		for (int i =0; i < CSV_HEADERS.length; i++) {
			writer.write(CSV_HEADERS[i]);
			if (i < CSV_HEADERS.length - 1) {
				writer.write(",");
			}
		}
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) ((DefaultTreeModel) this.getModel()).getRoot();
		for (int i = 0; i < root.getChildCount(); i++) {
			DefaultMutableTreeNode child = (DefaultMutableTreeNode) root.getChildAt(i);
			appendTreeNodeToWriter(child, nodeProblemMap, 1, writer);
		}
	}
	
	public static void appendTreeNodeToWriter(DefaultMutableTreeNode node, Map<DefaultMutableTreeNode, ProblemSentenceObject> nodeProblemMap, int level, Writer writer) throws IOException {
		ProblemSentenceObject problem = nodeProblemMap.get(node);
		if (problem != null) {
			if (!problem.getActivityPart().isEmpty() && !problem.getAssignmentPart().isEmpty() && !problem.getViolationPart().isEmpty()) {
				writer.write(lineSeparator);
				String violationPart = problem.getActivityPart()
						+ problem.getViolationPart();
				writer.write(violationPart);
				writer.write(",");
				writer.write(problem.getAttribute());
				writer.write(",");
				writer.write(problem.getAttributeValueCaption());
				writer.write(",");
				writer.write(Integer.toString(problem.getFrequency()));
				writer.write(",");
				float score = Math.round((problem.getRelevanceScore() * 1000.0f)) / 1000.0f;
				writer.write(Float.toString(score));
				writer.write(",");
				if (problem.isViolationAntecedent()) {
					writer.write("no");
				} else {
					writer.write("yes");
				}
			}
		}
		for (int i = 0; i < node.getChildCount(); i++) {
			DefaultMutableTreeNode child = (DefaultMutableTreeNode) node.getChildAt(i);
			appendTreeNodeToWriter(child, nodeProblemMap, level + 1, writer);
		}
	}
}

class DasboardTreeCellRenderer implements TreeCellRenderer {

	private Map<Object, DasboardTreeCellRendererComponent> valueComponentMap;

	private Map<DefaultMutableTreeNode, ProblemSentenceObject> nodeProblemMap;

	private ImageIcon activityIcon;
	private ImageIcon assignmentIcon;

	private boolean allowTopSelection;

	public DasboardTreeCellRenderer(Map<DefaultMutableTreeNode, ProblemSentenceObject> nodeProblemMap,
			boolean allowTopSelection) {
		this.nodeProblemMap = nodeProblemMap;
		this.allowTopSelection = allowTopSelection;
		this.valueComponentMap = new HashMap<Object, DasboardTreeCellRendererComponent>();
		try {
			Thread thread = Thread.currentThread();
			ClassLoader classLoader = thread.getContextClassLoader();
			InputStream input = classLoader.getResourceAsStream("images/activity.png");
			this.activityIcon = new ImageIcon(ImageIO.read(input));
			input.close();
			input = classLoader.getResourceAsStream("images/context.png");
			this.assignmentIcon = new ImageIcon(ImageIO.read(input));
			input.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	@Override
	public Component getTreeCellRendererComponent(JTree tree, Object value, boolean isSelected, boolean expanded,
			boolean leaf, int row, boolean hasFocus) {
		//		super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
		DasboardTreeCellRendererComponent component = valueComponentMap.get(value);
		if (component == null) {
			component = new DasboardTreeCellRendererComponent(this.allowTopSelection);
			if (value == null)
				return component;
			if (this.nodeProblemMap.get(value) == null) {
				//			if (((DefaultMutableTreeNode) value).getUserObject().toString().equals("Root Node")) {
				component.setUserObject(null);
				//			component.setIcon(expanded ? component.getOpenIcon() : component.getClosedIcon());
			} else {
				component.setUserObject(this.nodeProblemMap.get(value));
				if (this.assignmentIcon != null && this.activityIcon != null) {
					if (component.getUserObject() != null) {
						if (component.getUserObject().getActivity() == null
								|| !component.getUserObject().isViolationAntecedent())
							component.setIcon(this.assignmentIcon);
						else
							component.setIcon(this.activityIcon);
					}
				}
			}
			valueComponentMap.put(value, component);
		}
		//		this.setText("<html>Multi<br/>line<br/>text</html>");
		component.setSelect(isSelected);
		component.setFocus(hasFocus);
		//		this.repaint();
		//		this.invalidate();
		return component;
	}
}

class DasboardTreeCellRendererComponent extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 3672767466152701469L;

	protected static final Color activityColor = new Color(Color.green.getRed(), Color.green.getGreen(),
			Color.green.getBlue(), 50);
	protected static final Color assignmentColor = new Color(Color.blue.getRed(), Color.blue.getGreen(),
			Color.blue.getBlue(), 50);
	protected static final Color logOnlyColor = new Color(Color.orange.getRed(), Color.orange.getGreen(),
			Color.orange.getBlue(), 50);
	protected static final Color modelOnlyColor = new Color(Color.magenta.getRed(), Color.magenta.getGreen(),
			Color.magenta.getBlue(), 50);
	protected static final Color dataViolationColor = new Color(Color.red.getRed(), Color.red.getGreen(),
			Color.red.getBlue(), 50);
	protected static final Color timeViolationColor = new Color(Color.red.getRed(), Color.red.getGreen(),
			Color.red.getBlue(), 50);

	protected static final Color inactiveBackgroundColor = new Color(60, 60, 60, 100);

	private static final int tooltipMaxLineLength = 100;

	protected ProblemSentenceObject userObject;
	protected boolean allowTopSelection;

	protected JLabel icon;
	protected JTextArea text;

	public DasboardTreeCellRendererComponent(boolean allowTopSelection) {
		this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		//		this.icon = new JLabel() {
		//			public void setBackground(Color color) {
		//		        if (color instanceof ColorUIResource)
		//		          color = null;
		//		        super.setBackground(color);
		//		      }
		//		};
		this.add(icon = new JLabel());
		this.add(Box.createHorizontalStrut(4));
		this.add(text = new JTextArea());
		Font font = GUIUtilities.defaultFont;
		this.text.setFont(font);

		this.setOpaque(false);
		this.setBackground(null);
		this.text.setOpaque(true);

		this.allowTopSelection = allowTopSelection;

		//		ToolTipManager.sharedInstance().registerComponent(this);
		//		ToolTipManager.sharedInstance().registerComponent(this.text);
	}

	//	protected void paintComponent(Graphics g) {
	//		//		reflowText();
	//		super.paintComponent(g);
	//		boolean b = true;
	//		if (b)
	//			return;
	//		if (this.userObject != null) {
	//			Color oldColor = g.getColor();
	//
	//			FontMetrics metrics = g.getFontMetrics();
	//			int zeroPos = this.getIcon().getIconWidth() + this.getIconTextGap();
	//			int startY = 0;
	//			int height = metrics.getHeight();
	//			int start;
	//			int length;
	//			if (this.userObject.getActivityPart().length() > 0) {
	//				start = this.getText().indexOf(this.userObject.getActivityPart());
	//				start = metrics.stringWidth(this.getText().substring(0, start));
	//
	//				length = metrics.stringWidth(this.userObject.getActivityPart());
	//				Color color = activityColor;
	//				g.setColor(color);
	//				g.fillRect(start + zeroPos, startY, length, height);
	//			}
	//			if (this.userObject.getAssignmentPart().length() > 0) {
	//				start = this.getText().indexOf(this.userObject.getAssignmentPart());
	//				start = metrics.stringWidth(this.getText().substring(0, start));
	//
	//				length = metrics.stringWidth(this.userObject.getAssignmentPart());
	//				Color color = assignmentColor;
	//				g.setColor(color);
	//				g.fillRect(start + zeroPos, startY, length, height);
	//			}
	//			if (this.userObject.getViolationPart().length() > 0) {
	//				start = this.getText().indexOf(this.userObject.getViolationPart());
	//				start = metrics.stringWidth(this.getText().substring(0, start));
	//
	//				length = metrics.stringWidth(this.userObject.getViolationPart());
	//				Color color = null;
	//				if (this.userObject.getViolationClass() == 0) {
	//					color = logOnlyColor;
	//				} else if (this.userObject.getViolationClass() == 1) {
	//					color = modelOnlyColor;
	//				} else if (this.userObject.getViolationClass() == 2) {
	//					color = dataViolationColor;
	//				} else if (this.userObject.getViolationClass() == 3) {
	//					color = timeViolationColor;
	//				}
	//				g.setColor(color);
	//				g.fillRect(start + zeroPos, startY, length, height);
	//			}
	//			g.setColor(oldColor);
	//		}
	//	}

	public void setIcon(ImageIcon icon) {
		this.icon.setIcon(icon);
	}

	public void setUserObject(ProblemSentenceObject userObject) {
		this.userObject = userObject;
		if (userObject != null) {
			int toolTipLineLength = 0;
			StringBuilder toolTipBuilder = new StringBuilder();
			toolTipBuilder.append("<html>Cases: ");
			for (int i = 0; i < userObject.getInvolvedCases().size(); i++) {
				toolTipBuilder.append(userObject.getInvolvedCases().get(i));
				toolTipLineLength += userObject.getInvolvedCases().get(i).length();
				if (i < userObject.getInvolvedCases().size() - 1) {
					toolTipBuilder.append(",");
					if (toolTipLineLength >= tooltipMaxLineLength) {
						toolTipBuilder.append("<br/>");
						toolTipLineLength = 0;
					}
				}
			}
			toolTipBuilder.append("</html>");
			this.setToolTipText(toolTipBuilder.toString());
			//			this.text.setToolTipText(toolTipBuilder.toString());

			this.text.setText(GUIUtilities.reflowText(userObject.toString(), 600));
			System.out.println(this.text.getText());
			try {
				if (!this.allowTopSelection) {
					if (userObject.getViolationClass() == -1) {
						DefaultHighlighter.DefaultHighlightPainter highlighter = new DefaultHighlighter.DefaultHighlightPainter(
								inactiveBackgroundColor);
						this.text.setText(this.text.getText() + "*");
						this.text.getHighlighter().addHighlight(0, this.text.getText().length() - 1, highlighter);
					}
				}

				if (this.userObject.getActivityPart().length() > 0) {
					DefaultHighlighter.DefaultHighlightPainter highlighter = new DefaultHighlighter.DefaultHighlightPainter(
							activityColor);
					Pattern pattern = Pattern.compile(this.createPattern(userObject.getActivityPart()));
					Matcher matcher = pattern.matcher(this.text.getText());
					if (matcher.find()) {
						this.text.getHighlighter().addHighlight(matcher.start(), matcher.end(), highlighter);
					}
				}
				if (this.userObject.getAssignmentPart().length() > 0) {
					DefaultHighlighter.DefaultHighlightPainter highlighter = new DefaultHighlighter.DefaultHighlightPainter(
							assignmentColor);
					Pattern pattern = Pattern.compile(this.createPattern(userObject.getAssignmentPart()));
					Matcher matcher = pattern.matcher(this.text.getText());
					if (matcher.find()) {
						this.text.getHighlighter().addHighlight(matcher.start(), matcher.end(), highlighter);
					}
				}

				if (this.userObject.getViolationPart().length() > 0) {
					Color color = null;
					if (this.userObject.getViolationClass() == 0) {
						color = logOnlyColor;
					} else if (this.userObject.getViolationClass() == 1) {
						color = modelOnlyColor;
					} else if (this.userObject.getViolationClass() == 2) {
						color = dataViolationColor;
					} else if (this.userObject.getViolationClass() == 3) {
						color = timeViolationColor;
					}

					DefaultHighlighter.DefaultHighlightPainter highlighter = new DefaultHighlighter.DefaultHighlightPainter(
							color);
					Pattern pattern = Pattern.compile(this.createPattern(userObject.getViolationPart()));
					Matcher matcher = pattern.matcher(this.text.getText());
					if (matcher.find()) {
						this.text.getHighlighter().addHighlight(matcher.start(), matcher.end(), highlighter);
					}
				}
			} catch (BadLocationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	protected String createPattern(String original) {
		original = Pattern.quote(original);
		String pattern = original.replace(" ", "\\E( |\\r\\n){1,2}\\Q");
		//		String pattern = original.replace("(", "\\(").replace(" ", "( |\\r\\n){1,2}");
		return pattern;
	}

	public ProblemSentenceObject getUserObject() {
		return this.userObject;
	}

	public void setSelect(boolean isSelected) {
		Color bColor;
		if (isSelected) {
			bColor = UIManager.getColor("Tree.selectionBackground");
		} else {
			bColor = UIManager.getColor("Tree.textBackground");
		}
		this.text.setBackground(bColor);
	}

	public void setFocus(boolean hasFocus) {
		if (hasFocus) {
			Color lineColor = UIManager.getColor("Tree.selectionBorderColor");
			text.setBorder(BorderFactory.createLineBorder(lineColor));
		} else {
			text.setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));
		}
	}
}