package org.processmining.plugins.complianceframework.GUI;

import java.awt.Component;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.processmining.plugins.complianceframework.model.ComplianceDashboard;

public class DashboardTableCellRenderer extends DefaultTableCellRenderer {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private Map<TableColumn, java.awt.Color> columnColorMap;
	private static final int maxColorIntensity = 150;
	private static final java.awt.Color fullIncorrectColor = new java.awt.Color(maxColorIntensity, 0, 0);
	private static final java.awt.Color newtralColor = new java.awt.Color(75, 75, 75);

	public DashboardTableCellRenderer() {
		super();
	}

	public DashboardTableCellRenderer(JTable table, ComplianceDashboard dashboard) {
		this();
		init(table, dashboard);
	}

	public void init(JTable table, ComplianceDashboard dashboard) {
		TableColumnModel tcm = table.getColumnModel();
		columnColorMap = new HashMap<TableColumn, java.awt.Color>();
		// column with row names
		columnColorMap.put(tcm.getColumn(0), new java.awt.Color(75, 75, 75));
		columnColorMap.put(tcm.getColumn(1), new java.awt.Color(80, 80, 80));
		int maxEls = dashboard.getCombinations().get(dashboard.getCombinations().size() - 1).size();
		float intensity = (0.5f / (float) maxEls) * maxColorIntensity;
		// sync moves
		columnColorMap.put(tcm.getColumn(2), new java.awt.Color(0, Math.round(intensity), 0));
		// sync moves with correct & incorrect params
		for (int i = 0; i < dashboard.getCombinations().size(); i++) {
			intensity = ((float) dashboard.getCombinations().get(i).size() / (float) maxEls) * maxColorIntensity;
			columnColorMap.put(tcm.getColumn(i + 3), new java.awt.Color(0, Math.round(intensity), 0));
			columnColorMap.put(tcm.getColumn(dashboard.getCombinations().size() + i + 3),
					new java.awt.Color(Math.round(intensity), 0, 0));
		}
		// async moves
		columnColorMap.put(tcm.getColumn(table.getColumnCount() - 3), new java.awt.Color(maxColorIntensity, 0, 0));
		columnColorMap.put(tcm.getColumn(table.getColumnCount() - 2), new java.awt.Color(maxColorIntensity, 0, 0));
		columnColorMap.put(tcm.getColumn(table.getColumnCount() - 1), new java.awt.Color(75, 75, 75));
	}

	@Override
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus,
			int row, int column) {
		Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
		// TableColumn currentColumn = table.getColumn(
		// table.getColumnName(column));
		TableColumn currentColumn = table.getColumnModel().getColumn(column);

		if (value.toString().equals("0")) {
			c.setBackground(newtralColor);
		} else if (columnColorMap.containsKey(currentColumn)) {
			c.setBackground(columnColorMap.get(currentColumn));
		} 
		//Should remove
		else {
			if (column == 0) {
				c.setBackground(newtralColor);
			} else if (value.toString().equals("0")) {
				c.setBackground(newtralColor);
			} else if (column >= table.getColumnCount() - 3) {
				c.setBackground(fullIncorrectColor);
			} else {
				float intensity = 0.0f;
				int middle = (table.getColumnCount() - 5) / 2;
				try {
					if (column > middle + 1) {
						intensity = (float) (column - (middle + 1)) / ((float) middle);
						c.setBackground(new java.awt.Color(Math.round(maxColorIntensity * intensity), 0, 0));
					} else {
						intensity = (float) (column - 1) / (float) middle;
						c.setBackground(new java.awt.Color(0, Math.round(maxColorIntensity * intensity), 0));
					}
				} catch (Exception ex) {
					int a = 5;
					int b = a + 5;
					ex.printStackTrace();
				}
			}
			columnColorMap.put(currentColumn, c.getBackground());
		}

		return c;
	}
}