package org.processmining.plugins.complianceframework.GUI;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.JViewport;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;

import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;
import org.processmining.framework.util.ui.widgets.ProMScrollPane;
import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.plugins.complianceframework.model.InstanceClassifier;

import com.fluxicon.slickerbox.components.NiceIntegerSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.components.SlickerTabbedPane;
import com.fluxicon.slickerbox.factory.SlickerFactory;

import csplugins.id.mapping.ui.CheckComboBox;

public class TreeSettingsPanel extends JPanel {
	public CheckComboBox activtiyAttributeCCBox = null;
	public CheckComboBox assignmentAttributeCCBox = null;
	public CheckComboBox intersectionAttributeCCBox = null;

	public NiceIntegerSlider activityInstances4Leaf = null;
	public NiceIntegerSlider assignmentInstances4Leaf = null;
	public NiceIntegerSlider intersectionInstances4Leaf = null;

	public JCheckBox activityUnprunedTree = null;
	public JCheckBox assignmentUnprunedTree = null;
	public JCheckBox intersectionUnprunedTree = null;

	public JCheckBox activityBinarySplits = null;
	public JCheckBox assignmentBinarySplits = null;
	public JCheckBox intersectionBinarySplits = null;
	//	
	//	public NiceDoubleSlider activityPruningConfidence = null;
	//	public NiceDoubleSlider assignmentPruningConfidence = null;
	//	public NiceDoubleSlider intersectionPruningConfidence = null;

	public int activityTimeMode = 1;
	public String activityAnchor = null;
	public int assignmentTimeMode = 1;
	public String assignmentAnchor = null;
	public int intersectionTimeMode = 1;
	public String intersectionAnchor = null;
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private JTree tree;

	private String helpText = "In order to show, which data, resource and time values lead to the specified problem, the plugin needs to compare events with the problem and without it. In this screen you are offered to choose which attributes should be considered during the comparing.";

	public TreeSettingsPanel(String title, InstanceClassifier classifier, List<String> allAttributes,
			List<String> activityDefaultAttributes, List<String> assignmentDefaultAttributes,
			List<String> intersectionDefaultAttributes, String[] allActivities) {
		//		super(new BorderLayout());
		super();
		BoxLayout layout = new BoxLayout(this, BoxLayout.Y_AXIS);
		this.setLayout(layout);
		this.setFont(GUIUtilities.defaultFont);

		JLabel problemLabel = new JLabel(title);
		problemLabel.setHorizontalAlignment(SwingConstants.CENTER);
		problemLabel.setAlignmentX(CENTER_ALIGNMENT);
		problemLabel.setFont(GUIUtilities.defaultFont);
		JTextArea helpArea = new JTextArea(helpText);
		helpArea.setWrapStyleWord(true);
		helpArea.setLineWrap(true);
		helpArea.setEditable(false);
		helpArea.setFont(GUIUtilities.defaultFont);
		//		Dimension size = helpArea.getPreferredSize();
		//		size.height = 0;
		//		helpArea.setPreferredSize(size);
		this.add(problemLabel);
		this.add(helpArea);
		this.init(classifier, allAttributes, activityDefaultAttributes, assignmentDefaultAttributes,
				intersectionDefaultAttributes, allActivities);
		//		this.setupProperties(classifier, allAttributes, activityDefaultAttributes, assignmentDefaultAttributes,
		//				intersectionDefaultAttributes, allActivities);
	}

	public void init(InstanceClassifier classifier, List<String> allAttributes, List<String> activityDefaultAttributes,
			List<String> assignmentDefaultAttributes, List<String> intersectionDefaultAttributes, String[] allActivities) {
		//		JTabbedPane settingsTabbedPane = new JTabbedPane();
		SlickerTabbedPane settingsTabbedPane = SlickerFactory.instance().createTabbedPane("",
				WidgetColors.COLOR_LIST_BG, WidgetColors.COLOR_LIST_FG, Color.GREEN);
		//				Dimension size = new Dimension(settingsTabbedPane.getPreferredSize());
		//				size.height = Short.MAX_VALUE;
		//				settingsTabbedPane.setPreferredSize(size);
		this.add(settingsTabbedPane);
		ProMPropertiesPanel innerPanel = null;
		if (activityDefaultAttributes != null && activityDefaultAttributes.size() > 0
				&& classifier.activityClassificationInstanceCount > 0) {
			ProMPropertiesPanel activityPanel = setupProperties(classifier.activityClassificationInstanceCount,
					allAttributes, activityDefaultAttributes, allActivities, 0);
			settingsTabbedPane.addTab("Same activity", activityPanel);
			innerPanel = activityPanel;
			//			size.height = activityPanel.getPreferredSize().height;
		}
		if (assignmentDefaultAttributes != null && assignmentDefaultAttributes.size() > 0) {
			ProMPropertiesPanel assignmentPanel = setupProperties(classifier.assignmentClassificationInstanceCount,
					allAttributes, assignmentDefaultAttributes, allActivities, 1);
			settingsTabbedPane.addTab("Same attribute value", assignmentPanel);
			innerPanel = assignmentPanel;
			//			size.height = assignmentPanel.getPreferredSize().height;
		}
		if (intersectionDefaultAttributes != null && intersectionDefaultAttributes.size() > 0) {
			ProMPropertiesPanel intersectionPanel = setupProperties(classifier.intersectionClassificationInstanceCount,
					allAttributes, intersectionDefaultAttributes, allActivities, 2);
			settingsTabbedPane.addTab("Same activity and attribute value", intersectionPanel);
		}

		this.setBorder(new EmptyBorder(15, 15, 15, 15));
		//		this.setPreferredSize(size);
		this.addAncestorListener(this.createAncestorListener(settingsTabbedPane, innerPanel));
	}

	public ProMPropertiesPanel setupProperties(int instanceCount, List<String> allAttributes,
			List<String> defaultAttributes, String[] allActivities, int mode) {
		Color backColor = new Color(60, 60, 60, 160);
		String title = "Comparing moves with the same";
		Map<Object, Boolean> mapObjectsSelected = new HashMap<Object, Boolean>();
		for (String attribute : allAttributes) {
			mapObjectsSelected.put(attribute, defaultAttributes.contains(attribute));
		}
		CheckComboBox attributeCCBox = new CheckComboBox(mapObjectsSelected);
		NiceIntegerSlider instances4Leaf = SlickerFactory.instance().createNiceIntegerSlider("", 1, instanceCount, 2>instanceCount? instanceCount: 2,
				Orientation.HORIZONTAL);
		instances4Leaf.remove(0);
		instances4Leaf
				.setToolTipText("Minimal number of moves to be assigned to the same type (problematic or non-problematic)");
		JPanel timeSettingsPanel = this.createTimeSettingsPanel(allActivities, mode);
		timeSettingsPanel.setBackground(backColor);
		JCheckBox unprunnedTree = SlickerFactory.instance().createCheckBox("", false);
		JCheckBox binarySplits = SlickerFactory.instance().createCheckBox("", false);

		if (mode == 0) {
			this.activityInstances4Leaf = instances4Leaf;
			this.activtiyAttributeCCBox = attributeCCBox;
			this.activityBinarySplits = binarySplits;
			this.activityUnprunedTree = unprunnedTree;
			//			attributesLabel.setText(attributesLabel.getText() + " activity");
			title += " activity";
		} else if (mode == 1) {
			this.assignmentInstances4Leaf = instances4Leaf;
			this.assignmentAttributeCCBox = attributeCCBox;
			this.assignmentBinarySplits = binarySplits;
			this.assignmentUnprunedTree = unprunnedTree;
			//			attributesLabel.setText(attributesLabel.getText() + " attribute value");
			title += " attribute value";
		} else if (mode == 2) {
			this.intersectionInstances4Leaf = instances4Leaf;
			this.intersectionAttributeCCBox = attributeCCBox;
			this.intersectionBinarySplits = binarySplits;
			this.intersectionUnprunedTree = unprunnedTree;
			//			attributesLabel.setText(attributesLabel.getText() + " activity and attribute value");
			title += " activity and attribute value";
		}

		ProMPropertiesPanel result = new ProMPropertiesPanel(title);
		JViewport viewPort = ((ProMScrollPane) result.getComponent(2)).getViewport();
		JPanel resultContentPanel = (JPanel) viewPort.getView();

		//		JPanel attributesPanel = new JPanel();
		//		BoxLayout attrLayout = new BoxLayout(attributesPanel, BoxLayout.Y_AXIS);
		//		attributesPanel.setLayout(attrLayout);
		//		JLabel attributesLabel = SlickerFactory.instance().createLabel(
		//				"Attributes to consider for comparing moves with the same");
		//		attributesLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
		//		attributeCCBox.setAlignmentX(Component.RIGHT_ALIGNMENT);
		//		attributesPanel.add(attributesLabel);
		//		attributesLabel.add(Box.createVerticalStrut(20));
		//		attributesPanel.add(attributeCCBox);
		//		result.addProperty("", attributesLabel);
		result.addProperty("Considered attributes", attributeCCBox);
		((JPanel) (resultContentPanel.getComponent(resultContentPanel.getComponentCount() - 1))).getComponent(1)
		.setPreferredSize(new Dimension(200, 30));
		((JPanel) (resultContentPanel.getComponent(resultContentPanel.getComponentCount() - 1))).getComponent(1)
		.setMaximumSize(new Dimension(200, 1000));
		//		result.addProperty("", attributesPanel);
		//		attributesPanel.setOpaque(false);
		//		attributesPanel.setBackground(backColor);

		result.addProperty("Minimal number of events per class", instances4Leaf);
		//access label of the last property
		((JPanel) (resultContentPanel.getComponent(resultContentPanel.getComponentCount() - 1))).getComponent(1)
				.setPreferredSize(new Dimension(500, 30));
		((JPanel) (resultContentPanel.getComponent(resultContentPanel.getComponentCount() - 1))).getComponent(1)
				.setMaximumSize(new Dimension(500, 1000));
		result.addProperty("Considering time", timeSettingsPanel);
		
		resultContentPanel.add(Box.createVerticalStrut(3));
		resultContentPanel.add(this.createSubTitle("Advanced settings"));
//		result.addProperty("Advanced settings", new JLabel(""));
//		((JPanel) (resultContentPanel.getComponent(resultContentPanel.getComponentCount() - 1))).setBackground(WidgetColors.PROPERTIES_BACKGROUND);
//		((JPanel) (resultContentPanel.getComponent(resultContentPanel.getComponentCount() - 1))).setOpaque(true);
//		((JPanel) (resultContentPanel.getComponent(resultContentPanel.getComponentCount() - 1))).
		
		result.addProperty("Use unpruned tree", unprunnedTree);
		result.addProperty("Use binary splits for nominal attributes", binarySplits);
		//access label of the last property
		((JPanel) (resultContentPanel.getComponent(resultContentPanel.getComponentCount() - 1))).getComponent(1)
				.setPreferredSize(new Dimension(500, 30));
		((JPanel) (resultContentPanel.getComponent(resultContentPanel.getComponentCount() - 1))).getComponent(1)
				.setMaximumSize(new Dimension(500, 1000));

		GUIUtilities.setFont(result);
		GUIUtilities.setForeground(result, Color.black);
		return result;
	}

	public JPanel createTimeSettingsPanel(String[] allActivities, final int treeType) {
		ButtonGroup buttonGroup = new ButtonGroup();
		final JComboBox anchorComboBox = SlickerFactory.instance().createComboBox(allActivities);
		anchorComboBox.setVisible(false);

		JRadioButton disableRadioButton = SlickerFactory.instance().createRadioButton("Disable");
		disableRadioButton.setToolTipText("Time attribute will be ignored");
		disableRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JRadioButton sender = (JRadioButton) arg0.getSource();
				if (sender.isSelected()) {
					if (treeType == 0)
						activityTimeMode = 0;
					else if (treeType == 1)
						assignmentTimeMode = 0;
					else if (treeType == 2)
						intersectionTimeMode = 0;
					anchorComboBox.setVisible(false);
				}
			}
		});

		JRadioButton traceStartRadioButton = SlickerFactory.instance().createRadioButton("Start of the trace");
		traceStartRadioButton.setSelected(true);
		traceStartRadioButton
				.setToolTipText("Time between start of the trace and problematic activity will  be considered");
		traceStartRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JRadioButton sender = (JRadioButton) arg0.getSource();
				if (sender.isSelected()) {
					if (treeType == 0)
						activityTimeMode = 1;
					else if (treeType == 1)
						assignmentTimeMode = 1;
					else if (treeType == 2)
						intersectionTimeMode = 1;
					anchorComboBox.setVisible(false);
				}
			}
		});

		JRadioButton prevActivityRadioButton = SlickerFactory.instance().createRadioButton("Prevoius activity");
		prevActivityRadioButton
				.setToolTipText("Time between problematic activity and the one before it will  be considered");
		prevActivityRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JRadioButton sender = (JRadioButton) arg0.getSource();
				if (sender.isSelected()) {
					if (treeType == 0)
						activityTimeMode = 2;
					else if (treeType == 1)
						assignmentTimeMode = 2;
					else if (treeType == 2)
						intersectionTimeMode = 2;
					anchorComboBox.setVisible(false);
				}
			}
		});

		JRadioButton chosenActivityRadioButton = SlickerFactory.instance().createRadioButton("Chosen activity");
		chosenActivityRadioButton
				.setToolTipText("Time between problematic activity and the activity OF user choice will  be considered");
		chosenActivityRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JRadioButton sender = (JRadioButton) arg0.getSource();
				if (sender.isSelected()) {
					if (treeType == 0)
						activityTimeMode = 3;
					else if (treeType == 1)
						assignmentTimeMode = 3;
					else if (treeType == 2)
						intersectionTimeMode = 3;
					anchorComboBox.setVisible(true);
				} else
					anchorComboBox.setVisible(false);
			}
		});

		buttonGroup.add(disableRadioButton);
		buttonGroup.add(traceStartRadioButton);
		buttonGroup.add(prevActivityRadioButton);
		buttonGroup.add(chosenActivityRadioButton);

		JPanel timeSettingsPanel = new JPanel();
		BoxLayout timeSettingsLayout = new BoxLayout(timeSettingsPanel, BoxLayout.Y_AXIS);
		timeSettingsPanel.setLayout(timeSettingsLayout);

		JPanel radioPanel = new JPanel();
		GridLayout radioLayout = new GridLayout(2, 2);
		radioPanel.setLayout(radioLayout);
		radioPanel.add(disableRadioButton);
		radioPanel.add(traceStartRadioButton);
		radioPanel.add(prevActivityRadioButton);
		radioPanel.add(chosenActivityRadioButton);
		anchorComboBox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				JComboBox sender = (JComboBox) e.getSource();
				if (treeType == 0)
					activityAnchor = sender.getSelectedItem().toString();
				else if (treeType == 1)
					assignmentAnchor = sender.getSelectedItem().toString();
				else if (treeType == 2)
					intersectionAnchor = sender.getSelectedItem().toString();
			}
		});

		//		JLabel headerLabel = new JLabel("Considering time");
		//		timeSettingsPanel.add(headerLabel);
		timeSettingsPanel.add(radioPanel);
		timeSettingsPanel.add(anchorComboBox);
		radioPanel.setBackground(new Color(60, 60, 60, 160));
		radioPanel.setOpaque(false);
		timeSettingsPanel.setOpaque(false);
		return timeSettingsPanel;
	}

	private AncestorListener createAncestorListener(final JPanel outerPanel, final JPanel innerPanel) {
		//		Dimension outerSize = new Dimension(outerPanel.getPreferredSize());
		//		Dimension innerSize = new Dimension(innerPanel.getPreferredSize());
		//		int difference = (int) (outerSize.getHeight() - innerSize.getHeight() + 100);
		//		outerSize.height += difference;
		//		outerPanel.setPreferredSize(outerSize);
		//		outerPanel.setMinimumSize(outerSize);

		AncestorListener listener = new AncestorListener() {

			public void ancestorRemoved(AncestorEvent event) {
				// TODO Auto-generated method stub

			}

			public void ancestorMoved(AncestorEvent event) {
				// TODO Auto-generated method stub

			}

			public void ancestorAdded(AncestorEvent arg0) {
				if (!TreeSettingsPanel.this.isShowing())
					return;
				Dimension outerSize = new Dimension(outerPanel.getPreferredSize());
				//				Dimension innerSize = new Dimension(innerPanel.getPreferredSize());
				//				int difference = (int) ((innerSize.getHeight() + 100) - outerSize.getHeight());
				//				outerSize.height += 50;
				//				outerPanel.setPreferredSize(outerSize);
				//				outerPanel.setMinimumSize(outerSize);

				TreeSettingsPanel.this.adjustPrefferedSize();
			}
		};
		return listener;
	}

	public void adjustPrefferedSize() {
		int prefHeight = 0;
		for (int i = 0; i < this.getComponentCount(); i++) {
			prefHeight = (int) this.getComponent(i).getPreferredSize().getHeight();
		}
		prefHeight += 50;
		Dimension thisSize = TreeSettingsPanel.this.getPreferredSize();
		thisSize.height = prefHeight;
		thisSize.width += 20;
		this.setPreferredSize(thisSize);
		this.setMinimumSize(thisSize);
	}
	//	public JScrollPane createScrollPane(JTree tree) {
	//
	//		final JScrollPane scrollPane = new JScrollPane(tree);
	//		// final JScrollPane scrollPane = new JScrollPane(component);
	//		scrollPane.setOpaque(false);
	//		scrollPane.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
	//		scrollPane.getViewport().setOpaque(false);
	//		scrollPane.getViewport().setBackground(
	//				WidgetColors.PROPERTIES_BACKGROUND);
	//		scrollPane.setBorder(BorderFactory.createEmptyBorder());
	//		scrollPane
	//				.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
	//		scrollPane
	//				.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
	//		JScrollBar vBar = scrollPane.getVerticalScrollBar();
	//		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0),
	//				new Color(160, 160, 160), WidgetColors.COLOR_NON_FOCUS, 4, 12));
	//		vBar.setOpaque(false);
	//		vBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
	//		vBar = scrollPane.getHorizontalScrollBar();
	//		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0),
	//				new Color(160, 160, 160), WidgetColors.COLOR_NON_FOCUS, 4, 12));
	//		vBar.setOpaque(false);
	//		vBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
	//
	//		return scrollPane;
	//	}
	protected JPanel createSubTitle(final String name) {
//		final RoundedPanel packed = new RoundedPanel(10, 0, 0);
		final JPanel packed = new JPanel();
		packed.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
//		installHighlighter(component, target);
		packed.setLayout(new BoxLayout(packed, BoxLayout.X_AXIS));
		final JLabel nameLabel = new JLabel(name);
		nameLabel.setOpaque(false);
		nameLabel.setMinimumSize(new Dimension(150, 20));
		nameLabel.setMaximumSize(new Dimension(150, 1000));
		nameLabel.setPreferredSize(new Dimension(150, 30));

		packed.add(Box.createHorizontalStrut(5));
		packed.add(nameLabel);
		packed.add(Box.createHorizontalGlue());
//		packed.add(component);
//		packed.add(Box.createHorizontalStrut(5));
		packed.revalidate();
		return packed;
	}
}
