package org.processmining.plugins.complianceframework;

import java.text.ParseException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetFactory;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PetriNetWithData;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PetriNetWithDataFactory;
import org.processmining.models.guards.Expression;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.compliance.AbstractLogForCompliance_Plugin;
import org.processmining.plugins.complianceframework.GUI.DashboardTemporalPatternPanel;
import org.processmining.plugins.complianceframework.model.DashboardResultReplay;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.PNRepResultImpl;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;

public class TemporalReplayer {

	public static final String T_START = "Start";
	public static final String T_END = "End";
	public static final String T_INSTANCE_START = AbstractLogForCompliance_Plugin.INSTANCE_START_TRANSITION;
	public static final String T_INSTANCE_COMPLETE = AbstractLogForCompliance_Plugin.INSTANCE_COMPLETE_TRANSITION;
	public static final String T_OTHER = "other";

	public static DashboardResultReplay checkTemporalCompliance(UIPluginContext context, PNRepResult replayResult,
			XLog abstractedLog, Map<String, String> evClassTransMap) {

		Object[] createReplayOutput = createReplayLog(replayResult, abstractedLog, evClassTransMap);
		XLog replayLog = (XLog) createReplayOutput[0];
		PNRepResult newReplayResult = (PNRepResult) createReplayOutput[1];

		//		CreateTemporalPattern_Plugin createTemporalPattern = new CreateTemporalPattern_Plugin();
		//		Object[] tp = createTemporalPattern.createTemporalPattern(context, replayLog);
		//		PetriNetWithData net = (PetriNetWithData) tp[0];
		//
		//		TemporalComplianceRequirement req = (TemporalComplianceRequirement) tp[3];
		DashboardTemporalPatternPanel temporalConfiguration = new DashboardTemporalPatternPanel(replayLog);
		if (context.showConfiguration("Create Temporal pattern", temporalConfiguration) == InteractionResult.CANCEL) {
			//TODO: cancel
			return null;
		}

		Map<String, String> eventAttrnameMap = new HashMap<String, String>();
		Map<String, String> eventGuardMap = new HashMap<String, String>();

		for (DashboardTemporalPatternPanel.EventConfigurationPanel eventConfiguaration : temporalConfiguration.pickedGuardPanelList) {
			eventAttrnameMap.put(eventConfiguaration.event.getSelectedItem().toString(), "var_"
					+ eventConfiguaration.event.getSelectedItem().toString().replace(" ", "_") + "_Time");
			if (!eventConfiguaration.guard.getText().isEmpty())
				eventGuardMap.put(eventConfiguaration.event.getSelectedItem().toString(),
						eventConfiguaration.guard.getText());
		}

		for (DashboardTemporalPatternPanel.EventConfigurationPanel eventConfiguaration : temporalConfiguration.predefinedGuardPanelList) {
			eventAttrnameMap.put(eventConfiguaration.name.getText(), "var_"
					+ eventConfiguaration.name.getText().replace(" ", "_") + "_Time");
			if (!eventConfiguaration.guard.getText().isEmpty())
				eventGuardMap.put(eventConfiguaration.name.getText(), eventConfiguaration.guard.getText());
		}

		for (Entry<String, String> eventGuardEntry : eventGuardMap.entrySet()) {
			for (DashboardTemporalPatternPanel.EventConfigurationPanel eventConfiguaration : temporalConfiguration.pickedGuardPanelList) {
				Matcher matcher = Pattern.compile(
						"((\\(| )|^)(" + eventConfiguaration.name.getText() + "('?))((\\)| )|$)").matcher(
						eventGuardEntry.getValue());
				if (matcher.find())
					eventGuardEntry.setValue(matcher.replaceAll("$1"
							+ eventAttrnameMap.get(eventConfiguaration.event.getSelectedItem().toString()) + "$4$5"));
			}
			for (DashboardTemporalPatternPanel.EventConfigurationPanel eventConfiguaration : temporalConfiguration.predefinedGuardPanelList) {
				Matcher matcher = Pattern.compile(
						"((\\(| )|^)(" + eventConfiguaration.name.getText() + "('?))((\\)| )|$)").matcher(
						eventGuardEntry.getValue());
				if (matcher.find())
					eventGuardEntry.setValue(matcher.replaceAll("$1"
							+ eventAttrnameMap.get(eventConfiguaration.name.getText()) + "$4$5"));
			}
		}

		XLog enrichedLog = enrichLog(replayLog, eventAttrnameMap);

		Map<String, String> varMapping = new HashMap<String, String>();
		for (String var : eventAttrnameMap.values())
			varMapping.put(var, var);
		Map<String, Expression> guardsMap = new HashMap<String, Expression>();
		try {
			for (Entry<String, String> guardEntry : eventGuardMap.entrySet())
				guardsMap.put(guardEntry.getKey(), new Expression(guardEntry.getValue()));
		} catch (ParseException e) {
			e.printStackTrace();
		}
		
		DashboardResultReplay temporalReplay = DPNReplayer.replayDPN(context, enrichedLog, newReplayResult, varMapping,
				guardsMap, null, true);

		//		EnrichLog_Plugin enrichLog = new EnrichLog_Plugin();
		//		XLog enrichedLog = enrichLog.enrichLog(context, abstractedLog, req);
		//
		//		TemporalAlignment replayer = new TemporalAlignment();
		//		ResultReplay result = replayer.plugin(context, net, enrichedLog);
		//		
		return temporalReplay;
	}

	public static Object[] createReplayLog(PNRepResult controFlowAlignment, XLog abstractedLog,
			Map<String, String> evClassTransMap) {
		XFactory f = XFactoryRegistry.instance().currentDefault();

		//		List<SyncReplayResult> newCfReplayList =  new ArrayList<SyncReplayResult>();

		XLog replayLog = f.createLog();
		for (int i = 0; i < abstractedLog.size(); i++) {
			replayLog.add(null);
		}

		XConceptExtension XConcept = XConceptExtension.instance();
		XConcept.assignName(replayLog, XConcept.extractName(abstractedLog));
		for (SyncReplayResult cfReplay : controFlowAlignment) {

			for (Integer i : cfReplay.getTraceIndex()) {
				XTrace trace = abstractedLog.get(i.intValue());
				XTrace replayTrace = f.createTrace(trace.getAttributes());
				int originalEvIndex = 0;

				for (int j = 0; j < cfReplay.getNodeInstance().size(); j++) {
					Object aligned = cfReplay.getNodeInstance().get(j);
					StepTypes stepType = cfReplay.getStepTypes().get(j);
					XEvent replayEvent;
					XEvent event = trace.get(originalEvIndex);
					if (stepType == org.processmining.plugins.petrinet.replayresult.StepTypes.L) {
						// log moves: object is an XEvent						
						replayEvent = f.createEvent(event.getAttributes());
						XConcept.assignName(replayEvent, evClassTransMap.get(XConcept.extractName(event)));
					} else {
						// model moves: object is a Transition
						replayEvent = f.createEvent(f.createAttributeMap());
						XConceptExtension.instance().assignName(replayEvent, aligned.toString());
					}
					if ((stepType == org.processmining.plugins.petrinet.replayresult.StepTypes.L
							|| stepType == org.processmining.plugins.petrinet.replayresult.StepTypes.LMGOOD || stepType == org.processmining.plugins.petrinet.replayresult.StepTypes.LMNOGOOD)
							&& (originalEvIndex < (trace.size() - 1)))
						originalEvIndex += 1;

					XTimeExtension.instance().assignTimestamp(replayEvent,
							XTimeExtension.instance().extractTimestamp(event));

					replayTrace.add(replayEvent);
				}
				replayLog.set(i.intValue(), replayTrace);
			}
		}

		// [HV] Not that nice, but I see no other way to do this.
		PNRepResult newControFlowAlignment = (PNRepResult) ((PNRepResultImpl) controFlowAlignment).clone();
		for (SyncReplayResult cfReplay : newControFlowAlignment) {
			for (int i = 0; i < cfReplay.getStepTypes().size(); i++)
				cfReplay.getStepTypes().set(i, org.processmining.plugins.petrinet.replayresult.StepTypes.LMGOOD);
		}

		return new Object[] { replayLog, newControFlowAlignment };
	}

	public static XLog enrichLog(XLog replayLog, Map<String, String> eventAttrnameMap) {
		XFactory xfactory = XFactoryRegistry.instance().currentDefault();
		XLog enrichedLog = xfactory.createLog(replayLog.getAttributes());
		for (XTrace replayTrace : replayLog) {
			XTrace enrichedTrace = xfactory.createTrace(replayTrace.getAttributes());
			for (XEvent replayEvent : replayTrace) {
				XEvent enrichedEvent = xfactory.createEvent(replayEvent.getAttributes());
				String attrName = eventAttrnameMap.get(XConceptExtension.instance().extractName(enrichedEvent));
				if (attrName != null) {
					Date timeStamp = XTimeExtension.instance().extractTimestamp(enrichedEvent);
					XAttributeTimestamp externalTimeStamp = xfactory.createAttributeTimestamp(attrName, timeStamp,
							XTimeExtension.instance());
					enrichedEvent.getAttributes().put(attrName, externalTimeStamp);
				}
				enrichedTrace.add(enrichedEvent);
			}
			enrichedLog.add(enrichedTrace);
		}

		return enrichedLog;
	}

	public static PetriNetWithData createTemporalPattern(PluginContext context, Map<String, String> eventAttrnameMap,
			Map<String, String> eventGuardMap) {

		// step 1: create low-level Petri net skeleton of temporal pattern
		Petrinet net = PetrinetFactory.newPetrinet("temporal pattern skeleton");

		Map<String, Transition> transitions = new HashMap<String, Transition>();

		Place p_initial = net.addPlace("initial");
		Place p_inactive = net.addPlace("inactive");
		Place p_active = net.addPlace("active");
		Place p_final = net.addPlace("final");

		Transition t_start = net.addTransition(T_START);
		net.addArc(p_initial, t_start);
		net.addArc(t_start, p_inactive);
		transitions.put(T_START, t_start);

		Transition t_end = net.addTransition(T_END);
		net.addArc(p_inactive, t_end);
		net.addArc(t_end, p_final);
		transitions.put(T_END, t_end);

		Transition t_I_start = net.addTransition(T_INSTANCE_START);
		net.addArc(p_inactive, t_I_start);
		net.addArc(t_I_start, p_active);
		transitions.put(T_INSTANCE_START, t_I_start);

		Transition t_I_complete = net.addTransition(T_INSTANCE_COMPLETE);
		net.addArc(p_active, t_I_complete);
		net.addArc(t_I_complete, p_inactive);
		transitions.put(T_INSTANCE_COMPLETE, t_I_complete);

		Transition t_omega_1 = net.addTransition(T_OTHER);
		net.addArc(p_inactive, t_omega_1);
		net.addArc(t_omega_1, p_inactive);
		Transition t_omega_2 = net.addTransition(T_OTHER);
		net.addArc(p_active, t_omega_2);
		net.addArc(t_omega_2, p_active);

		// create 1 transition for each specified event

		for (String name : eventAttrnameMap.keySet()) {
			//			if (isPredefinedName(name)) continue;
			Transition t_x = net.addTransition(name);
			net.addArc(p_active, t_x);
			net.addArc(t_x, p_active);

			transitions.put(name, t_x);
		}

		// step2: migrate low-level net to Petri net with variables
		PetriNetWithDataFactory f = new PetriNetWithDataFactory(net, "temporal pattern");
		PetriNetWithData temporal_net = f.getRetValue();

		// step 3: create initial and final marking
		Marking m_initial = new Marking();
		m_initial.add(f.getPlaceMapping().get(p_initial));
		context.addConnection(new InitialMarkingConnection(temporal_net, m_initial));

		Marking m_final = new Marking();
		m_final.add(f.getPlaceMapping().get(p_final));
		context.addConnection(new FinalMarkingConnection(temporal_net, m_final));

		// step 4: declare variables
		for (String varName : eventAttrnameMap.values()) {

			temporal_net.addVariable(varName, java.util.Date.class, null, null);
		}

		// step 5: write guards and set read and write operations on transitions
		try {

			// for each guard
			for (Entry<String, String> guardEntry : eventGuardMap.entrySet()) {

				System.out.println("guard: " + guardEntry.getValue());

				Expression guard = new Expression(guardEntry.getValue());

				// set guard
				temporal_net.setGuard(transitions.get(guardEntry.getKey()), guard);
			}

		} catch (Exception e) {
			System.err.println(e);
			e.printStackTrace();
		}

		return temporal_net;
	}
}
