package org.processmining.plugins.complianceframework.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.deckfour.xes.model.XLog;
import org.processmining.plugins.DataConformance.framework.ExecutionStep;
import org.processmining.plugins.DataConformance.framework.ReplayState;
import org.processmining.plugins.DataConformance.visualization.DataAwareStepTypes;

public class DashboardResultReplay implements Serializable {
	
	public static final String bottomActivityName = "\u03BB";
	/**
	 * 
	 */
	private static final long serialVersionUID = 2573433985087105278L;
	protected static float epsilon = 0.00005F;

	public Collection<DashboardAlignment> labelStepArray;
	public Map<String, String> variableMapping;

	//	public XLog alignedLog;

	public DashboardResultReplay() { }
	
	public DashboardResultReplay(Collection<? extends ReplayState> replayStateCollection, XLog alignedLog,
			Map<String, String> variableMapping) {
		//		this.alignedLog = alignedLog;
		this.variableMapping = variableMapping;

		this.labelStepArray = new ArrayList<DashboardAlignment>();
		for (ReplayState state : replayStateCollection) {
			Iterator<ExecutionStep> logTraceIter = state.getLogTracePrefix().iterator();
			Iterator<ExecutionStep> processTraceIter = state.getProcessTracePrefix().iterator();
			ArrayList<DataAwareStepTypes> stepArray = new ArrayList<DataAwareStepTypes>();
			DashboardAlignment align = new DashboardAlignment(state, 0.0f, null, stepArray);
			this.labelStepArray.add(align);
			while (logTraceIter.hasNext() && processTraceIter.hasNext()) {
				ExecutionStep logStep = logTraceIter.next();
				ExecutionStep processStep = processTraceIter.next();

				if (logStep.getActivity().equals(bottomActivityName)) {
					if (processStep.isInvisible())
						stepArray.add(DataAwareStepTypes.MINVI);
					else
						stepArray.add(DataAwareStepTypes.MREAL);
				} else if (processStep.getActivity().equals(bottomActivityName)) {
					stepArray.add(DataAwareStepTypes.L);
				} else {
					if (executionStepsWeakEqual(processStep, logStep)) {
						stepArray.add(DataAwareStepTypes.LMGOOD);
					} else
						stepArray.add(DataAwareStepTypes.LMNOGOOD);
				}
			}
		}
	}

	public static boolean executionStepsWeakEqual(ExecutionStep procStep, ExecutionStep logStep) {
		if (procStep.getActivity() == null) {
			return (logStep.getActivity() == null);
		} else {
			if (procStep.getActivity().equals(logStep.getActivity())) {
				for (Entry<String, Object> entry : procStep.entrySet()) {
					Object logValue = logStep.get(entry.getKey());
					Object procValue = entry.getValue();
					if (procValue != null && logValue == null)
						return false;
					else if (procValue == null && logValue != null)
						return false;
					else if (procValue instanceof Number || logValue instanceof Number) {
						double procNumber;
						if (procValue instanceof Number)
							procNumber = ((Number) procValue).doubleValue();
						else if (procValue instanceof String)
							try {
								procNumber = Double.parseDouble((String) procValue);
							} catch (NumberFormatException e) {
								return false;
							}
						else
							return false;
						double logNumber;
						if (logValue instanceof Number)
							logNumber = ((Number) logValue).doubleValue();
						else if (logValue instanceof String) {
							try {
								logNumber = Double.parseDouble((String) logValue);
							} catch (NumberFormatException e) {
								return false;
							}
						} else
							return false;
						if (procNumber < logNumber * (1 - epsilon) || procNumber > logNumber * (1 + epsilon))
							return false;
					} else {
						if (!logValue.equals(entry.getValue()))
							return false;
					}
				}
				return true;
			} else
				return false;
		}
	}
}
