package org.processmining.plugins.complianceframework.visualization;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Image;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.tree.DefaultMutableTreeNode;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.plugins.complianceframework.GUI.TreeDescriptionPanel;
import org.processmining.plugins.complianceframework.GUI.VennDiagramPanel;
import org.processmining.plugins.complianceframework.model.InstanceClassifier;
import org.processmining.plugins.complianceframework.model.ProblemSentenceObject;

import com.fluxicon.slickerbox.components.SlickerTabbedPane;
import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

public class ProblemInsightVisualizer {

	private Image checkImage = null;
	private Image crossImage = null;

	@Plugin(name = "Visualize problem insight", parameterLabels = { "Problem insight" }, returnLabels = { "Visualization" }, returnTypes = { JComponent.class }, userAccessible = false, help = "Visualize problem insight")
	@Visualizer
	public JComponent plugin(UIPluginContext context, InstanceClassifier classifier) throws Exception {
		SlickerTabbedPane tabbedPane = SlickerFactory.instance().createTabbedPane("", WidgetColors.COLOR_LIST_BG,
				WidgetColors.COLOR_LIST_FG, Color.GREEN);
		//		JTabbedPane tabbedPane = new JTabbedPane();

		Map[] descriptionMapAr = new Map[3];
		if (classifier.getActivityGraph() != null) {
			JPanel activityOuterPanel = new JPanel(new BorderLayout());
			DefaultMutableTreeNode activityTreeRootNode = InstanceClassifier.buildTree(classifier.getActivityGraph());
			JLabel titleLabel;
			titleLabel = new JLabel("Events with activity " + classifier.problemSentenceObject.getActivity());
			titleLabel.setFont(titleLabel.getFont().deriveFont(18.0f));
			titleLabel.setHorizontalAlignment(SwingConstants.CENTER);
			//			TreeContainerPanel activityTreeContainerPanel = new TreeContainerPanel("Activity tree", activityTree);
			//			activitySplitPane.setLeftComponent(activityTreeContainerPanel);
			Map<String, List<List<String>>> descriptionMap = new HashMap<String, List<List<String>>>();
			descriptionMapAr[0] = descriptionMap;
			for (String problem : classifier.getActivityClassValues()) {
				List<List<String>> preconditions = InstanceClassifier.findPreconditions(problem, activityTreeRootNode,
						classifier.activityTimeAttributeMode, classifier.getActivityInstances());
				if (preconditions != null)
					descriptionMap.put(problem, preconditions);
			}
			JScrollPane activityTreeDescriptionPanel = this.createTreeDescriptionPane(classifier.problemSentenceObject,
					descriptionMap);
			//			activitySplitPane.setRightComponent(activityTreeDescriptionPanel);
			activityOuterPanel.add(titleLabel, BorderLayout.NORTH);
			if (descriptionMap.keySet().size() > 0) {
				//				activityOuterPanel.add(activitySplitPane, BorderLayout.CENTER);
				activityOuterPanel.add(activityTreeDescriptionPanel, BorderLayout.CENTER);
			} else {
				createFailLabel(activityOuterPanel);
			}
			tabbedPane.addTab("Activity tree", activityOuterPanel);
		}

		if (classifier.getAssignmentGraph() != null) {
			JPanel assignmentOuterPanel = new JPanel(new BorderLayout());
			DefaultMutableTreeNode assignmentTreeRootNode = InstanceClassifier.buildTree(classifier.getAssignmentGraph());
			JLabel titleLabel;
			titleLabel = new JLabel("Events with " + classifier.problemSentenceObject.getAssignmentPartNoPrefix());
			titleLabel.setFont(titleLabel.getFont().deriveFont(18.0f));
			titleLabel.setHorizontalAlignment(SwingConstants.CENTER);
			Map<String, List<List<String>>> descriptionMap = new HashMap<String, List<List<String>>>();
			descriptionMapAr[1] = descriptionMap;
			for (String problem : classifier.getAssignmentClassValues()) {
				List<List<String>> preconditions = InstanceClassifier.findPreconditions(problem, assignmentTreeRootNode,
						classifier.assignmentTimeAttributeMode, classifier.getAssignmentInstances());
				if (preconditions != null)
					descriptionMap.put(problem, preconditions);
			}
			JScrollPane assignmentTreeDescriptionPanel = this.createTreeDescriptionPane(
					classifier.problemSentenceObject, descriptionMap);
			assignmentOuterPanel.add(titleLabel, BorderLayout.NORTH);
			if (descriptionMap.keySet().size() > 0) {
				assignmentOuterPanel.add(assignmentTreeDescriptionPanel, BorderLayout.CENTER);
			} else {
				createFailLabel(assignmentOuterPanel);
			}
			tabbedPane.addTab("Assignment tree", assignmentOuterPanel);
		}

		if (classifier.getIntersectionGraph() != null) {
			JPanel intersectionOuterPanel = new JPanel(new BorderLayout());
			//			ProMSplitPane intersectionSplitPane = new ProMSplitPane(ProMSplitPane.HORIZONTAL_SPLIT);
			DefaultMutableTreeNode intersectionTreeRootNode = InstanceClassifier.buildTree(classifier.getIntersectionGraph());
			//			JTree intersectionTree = new JTree(intersectionTreeRootNode);
			JLabel titleLabel;
			//			titleLabel = new JLabel(classifier.problemSentenceObject.toStringNoSeverity()
			//					+ "; Problematic moves compared to other moves with the same actvitiy");
			titleLabel = new JLabel("Events with actvitiy " + classifier.problemSentenceObject.getActivity() + " and "
					+ classifier.problemSentenceObject.getAssignmentPartNoPrefix());
			titleLabel.setFont(titleLabel.getFont().deriveFont(18.0f));
			titleLabel.setHorizontalAlignment(SwingConstants.CENTER);
			//			TreeContainerPanel intersectionTreeContainerPanel = new TreeContainerPanel("intersection tree",
			//					intersectionTree);
			//			intersectionSplitPane.setLeftComponent(intersectionTreeContainerPanel);
			Map<String, List<List<String>>> descriptionMap = new HashMap<String, List<List<String>>>();
			descriptionMapAr[2] = descriptionMap;
			for (String problem : classifier.getIntersectionClassValues()) {
				List<List<String>> preconditions = InstanceClassifier.findPreconditions(problem, intersectionTreeRootNode,
						classifier.intersectionTimeAttributeMode, classifier.getIntersectionInstances());
				if (preconditions != null)
					descriptionMap.put(problem, preconditions);
			}
			JScrollPane intersectionTreeDescriptionPanel = this.createTreeDescriptionPane(
					classifier.problemSentenceObject, descriptionMap);
			//			intersectionSplitPane.setRightComponent(intersectionTreeDescriptionPanel);
			intersectionOuterPanel.add(titleLabel, BorderLayout.NORTH);
			if (descriptionMap.keySet().size() > 0) {
				//				intersectionOuterPanel.add(intersectionSplitPane, BorderLayout.CENTER);
				intersectionOuterPanel.add(intersectionTreeDescriptionPanel, BorderLayout.CENTER);
			} else {
				createFailLabel(intersectionOuterPanel);
			}
			tabbedPane.addTab("Intersection tree", intersectionOuterPanel);
		}
		for (int i = 0; i < descriptionMapAr.length; i++)
			if (descriptionMapAr[i] == null)
				return tabbedPane;
		tabbedPane
				.addTab("Venn diagramm", new VennDiagramPanel("", classifier.problemSentenceObject, descriptionMapAr));

		return tabbedPane;
	}
	
	public JScrollPane createTreeDescriptionPane(ProblemSentenceObject problemSentenceObject,
			Map<String, List<List<String>>> descriptions) {
		if (descriptions == null)
			return null;
		this.loadImages();
		JPanel visualPanel = null;
		visualPanel = createVisualPanel(problemSentenceObject, descriptions);
		JScrollPane scrollPane = createScrollPane(visualPanel);
		return scrollPane;
	}

	private void createFailLabel(JPanel parent) {
		JLabel failLabel = new JLabel("Events are indistinguishable with current settings");
		failLabel.setFont(failLabel.getFont().deriveFont(18.0f));
		failLabel.setHorizontalAlignment(SwingConstants.CENTER);
		parent.add(failLabel, BorderLayout.CENTER);
	}

	public JPanel createVisualPanel(ProblemSentenceObject problemSentenceObject,
			Map<String, List<List<String>>> descriptions) {
		ProMSplitPane visualPanel = new ProMSplitPane(ProMSplitPane.HORIZONTAL_SPLIT);
		visualPanel.setResizeWeight(0.5);
		if (descriptions.get("non-problematic") != null)
			visualPanel.setLeftComponent(new TreeDescriptionPanel(this.checkImage, descriptions.get("non-problematic"),
					problemSentenceObject.getNonViolationPartForInsight()));
		if (descriptions.get("problematic") != null)
			visualPanel.setRightComponent(new TreeDescriptionPanel(this.crossImage, descriptions.get("problematic"),
					problemSentenceObject.getViolationPartForInsight()));
		return visualPanel;
	}

	public void loadImages() {
		try {
			Thread thread = Thread.currentThread();
			ClassLoader classLoader = thread.getContextClassLoader();
			//			InputStream input = new FileInputStream("images/Check.png");
			InputStream input = classLoader.getResourceAsStream("images/Check.png");
			this.checkImage = ImageIO.read(input);
			input.close();
			//			input = new FileInputStream("images/Cross.png");
			input = classLoader.getResourceAsStream("images/Cross.png");
			this.crossImage = ImageIO.read(input);
			input.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public JScrollPane createScrollPane(JPanel panel) {

		final JScrollPane scrollPane = new JScrollPane(panel);
		// final JScrollPane scrollPane = new JScrollPane(component);
		scrollPane.setOpaque(false);
		scrollPane.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		scrollPane.getViewport().setOpaque(false);
		scrollPane.getViewport().setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		JScrollBar vBar = scrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
				WidgetColors.COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		vBar = scrollPane.getHorizontalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
				WidgetColors.COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);

		return scrollPane;
	}
}
