package org.processmining.plugins.datacompliance;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.complianceframework.model.ComplianceExecutionStep;
import org.processmining.plugins.complianceframework.model.DashboardAlignment;
import org.processmining.plugins.complianceframework.model.DashboardResultReplay;

@Plugin(name = "Export Data-Aware replay result Dashboard as csv", returnLabels = {}, returnTypes = {}, parameterLabels = {
		"Data-aware Compliance result", "File" }, userAccessible = true)
@UIExportPlugin(description = "Export Data-Aware replay result file", extension = "csv")
public class DashboardResultReplayExportPlugin {
	private static String headers = "TRACE:concept:name,EVT:ruleMeasure,EVT:rulePrimaryKey,EVT:originalEventID,EVT:derivedTraceID,EVT:ruleName,EVT:ruleText,EVT:derivedEventID,EVT:requirementID,EVT:ruleType,EVT:originalLogID,EVT:Analyzing_Attribute,EVT:derivedLogID,ComplianceCheckingResultID,alignmentID,moveID,violationID,compliantValue,violated,ruleInstance";
	
	@PluginVariant(requiredParameterLabels = { 0, 1 })
	public void export(UIPluginContext context, DashboardResultReplay replayResult, File file) throws IOException {
		file.createNewFile();
		FileOutputStream fileOutputStream = new FileOutputStream(file, false);
		OutputStreamWriter outputStreamWriter = new OutputStreamWriter(fileOutputStream, Charset.forName("UTF-8"));
		BufferedWriter bufferedWriter = new BufferedWriter(outputStreamWriter);
		bufferedWriter.write(headers);
		bufferedWriter.newLine();
		for (DashboardAlignment alignment : replayResult.labelStepArray) {
			for (int i = 0; i < alignment.size(); i++) {
//				ComplianceExecutionStep procStep = (ComplianceExecutionStep)alignment.getProcessTrace().get(i);
				ComplianceExecutionStep logStep = (ComplianceExecutionStep)alignment.getLogTrace().get(i);
				bufferedWriter.write(alignment.getTraceName());
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.ruleMeasure);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.rulePrimaryKey);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.originalEventID);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.derivedTraceID);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.ruleName);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.ruleText);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.derivedEventID);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.requirementID);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.ruleType);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.originalLogID);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.Analyzing_Attribute);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.derivedLogID);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.ComplianceCheckingResultID);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.alignmentID);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.moveID);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.violationID);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.compliantValue);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.violated);
				bufferedWriter.write(",");
				bufferedWriter.write(logStep.ruleInstance);
				bufferedWriter.newLine();
			}
		}

		bufferedWriter.close();
	}
}
