package org.processmining.plugins.datacompliance;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeBooleanImpl;
import org.deckfour.xes.model.impl.XAttributeContinuousImpl;
import org.deckfour.xes.model.impl.XAttributeDiscreteImpl;
import org.deckfour.xes.model.impl.XAttributeLiteralImpl;
import org.deckfour.xes.model.impl.XAttributeTimestampImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.DataElement;
import org.processmining.models.guards.Expression;
import org.processmining.plugins.DataConformance.framework.ExecutionStep;
import org.processmining.plugins.DataConformance.visualization.DataAwareStepTypes;
import org.processmining.plugins.complianceframework.DPNReplayer;
import org.processmining.plugins.complianceframework.model.DashboardAlignment;
import org.processmining.plugins.complianceframework.model.DashboardResultReplay;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.datacompliance.GUI.ActivitiesSelectionPanel;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;

public class DataComplianceActivitiesPlugin {
	public static final String NAME_ATTRIBUTE_NAME = "Name";
	public static final String DATA_CONDITION_ATTR_NAME = "DataCondition";

	public static final String CONDITION_HOLDS_ATTR_NAME = "ConditionHolds";
	public static final String CONDITION_HOLDS_TRUE = "true";
	public static final String CONDITION_HOLDS_FALSE = "false";
	public static final String CONDITION_HOLDS_IRRELEVANT = ">>";

	public static final String NAME_AND_CONDITION_HOLDS_ATTR_NAME = "Name&ConditionHolds";

	@Plugin(name = "Data and Resource Compliance Checking-Rules Restricting Activities", parameterLabels = { "Log",
			"DP Net", "Petri Net" }, returnLabels = { "Compliance result" }, returnTypes = { PNRepResult.class })
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "V. Gromov", email = "v.gromov@hotmail.com")
	public PNRepResult checkCompliance(UIPluginContext context, XLog originalLog, DataPetriNet DPNet,
			PetrinetGraph petriNet) {
		context.getProgress().setIndeterminate(true);

		Set<String> logAttributesSet = new LinkedHashSet<String>();

		XLog modifiedLog = this.copyTraceAttributesToLog(originalLog, logAttributesSet);

		Map<XEventClassifier, XEventClass[]> map_classifierAvailableEvClasses = new LinkedHashMap<XEventClassifier, XEventClass[]>();
		map_classifierAvailableEvClasses.put(XLogInfoImpl.STANDARD_CLASSIFIER,
				XLogInfoImpl.create(modifiedLog, XLogInfoImpl.STANDARD_CLASSIFIER).getEventClasses().getClasses()
						.toArray(new XEventClass[1]));
		map_classifierAvailableEvClasses.put(
				XLogInfoImpl.NAME_CLASSIFIER,
				XLogInfoImpl.create(modifiedLog, XLogInfoImpl.NAME_CLASSIFIER).getEventClasses().getClasses()
						.toArray(new XEventClass[1]));
		map_classifierAvailableEvClasses.put(XLogInfoImpl.RESOURCE_CLASSIFIER,
				XLogInfoImpl.create(modifiedLog, XLogInfoImpl.RESOURCE_CLASSIFIER).getEventClasses().getClasses()
						.toArray(new XEventClass[1]));
		map_classifierAvailableEvClasses.put(XLogInfoImpl.LIFECYCLE_TRANSITION_CLASSIFIER,
				XLogInfoImpl.create(modifiedLog, XLogInfoImpl.LIFECYCLE_TRANSITION_CLASSIFIER).getEventClasses()
						.getClasses().toArray(new XEventClass[1]));
		ActivitiesSelectionPanel activitiesSelectionPanel = new ActivitiesSelectionPanel(
				map_classifierAvailableEvClasses);
		InteractionResult interactionResult = context.showConfiguration("Uncheck actifities to abstract from",
				activitiesSelectionPanel);
		if (interactionResult == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		Entry<XEventClassifier, Set<XEventClass>> selectedEventClasses = activitiesSelectionPanel
				.getSelectedActivities();
		//key - DPN variable; value - log attribute
		Map<String, String> variableMapping = DataComplianceHelper.getVariableMapping(context, logAttributesSet, DPNet);
		Entry<String, String> conditionAttribute;
		if (variableMapping == null) {
			context.getFutureResult(0).cancel(true);
			return null;
		} else {

			if (variableMapping.entrySet().iterator().hasNext())
				conditionAttribute = variableMapping.entrySet().iterator().next();
			else {
				context.getFutureResult(0).cancel(true);
				return null;
			}
		}

		Set<XEventClass> activitiesThatUpdateAttributesSet = new LinkedHashSet<XEventClass>(
				selectedEventClasses.getValue());
		activitiesThatUpdateAttributesSet.add(DataComplianceHelper.getOmegaClass(selectedEventClasses.getKey()));
		map_classifierAvailableEvClasses.clear();
		map_classifierAvailableEvClasses.put(selectedEventClasses.getKey(),
				activitiesThatUpdateAttributesSet.toArray(new XEventClass[1]));

		ActivitiesSelectionPanel activitiesThatUpdateAttributesSelectionPanel = new ActivitiesSelectionPanel(
				"Select activities that can update variables", map_classifierAvailableEvClasses, true);
		interactionResult = context.showConfiguration("Select activities that can update attribute values",
				activitiesThatUpdateAttributesSelectionPanel);
		if (interactionResult == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		activitiesThatUpdateAttributesSet = activitiesThatUpdateAttributesSelectionPanel.getSelectedActivities()
				.getValue();

		Map<String, Expression> guardsMap = DataComplianceHelper.getGuardsMap(DPNet);
		Map<String, Transition> transitionsMap = DataComplianceHelper.getTransitionsMap(DPNet);

		modifiedLog = this.prepareLogForDataReplay(modifiedLog, selectedEventClasses.getKey(),
				selectedEventClasses.getValue(), activitiesThatUpdateAttributesSet, conditionAttribute);

		XEventClassifier[] availableEventClass = new XEventClassifier[1];
		availableEventClass[0] = XLogInfoImpl.NAME_CLASSIFIER;
		Object[] dpnMappingOutput = DataComplianceHelper.getActivitiesMapping(context, modifiedLog, DPNet,
				availableEventClass);
		TransEvClassMapping dpn_Mapping = (TransEvClassMapping) dpnMappingOutput[0];

		if (dpn_Mapping == null) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		PNRepResult cfRepRes = DataComplianceHelper.cfReplayLog(context, DPNet, dpn_Mapping, modifiedLog);
		if (cfRepRes == null) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		context.getProvidedObjectManager().createProvidedObject(
				"Control-flow compliance alignment for " + XConceptExtension.instance().extractName(modifiedLog)
						+ " on " + DPNet.getLabel(), cfRepRes, PNRepResult.class, context);
		context.getProvidedObjectManager()
				.createProvidedObject(
						"Mapping between " + XConceptExtension.instance().extractName(modifiedLog) + " and "
								+ DPNet.getLabel(), dpn_Mapping, TransEvClassMapping.class, context);

		Map<String, Class<?>> variableTypes = new HashMap<>();
		for (DataElement variable : DPNet.getVariables()) {
			variableTypes.put(variable.getVarName(), variable.getType());
		}
		DashboardResultReplay dataRepRes = DPNReplayer.replayDPN(context, modifiedLog, cfRepRes, variableMapping,
				guardsMap, transitionsMap, variableTypes, false);

		modifiedLog = this.prepareLogForCFReplay(modifiedLog, dataRepRes, conditionAttribute);
		context.getProvidedObjectManager().createProvidedObject(
				"Modified log based on " + XConceptExtension.instance().extractName(originalLog), modifiedLog,
				XLog.class, context);

		Object[] pnMappingOutput = DataComplianceHelper.getActivitiesMapping(context, modifiedLog, petriNet,
				availableEventClass);
		if (pnMappingOutput == null) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		TransEvClassMapping pn_mapping = (TransEvClassMapping) pnMappingOutput[0];

		PNRepResult replayOutput = DataComplianceHelper.cfReplayLog(context, petriNet, pn_mapping, modifiedLog);
		if (replayOutput == null) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		return replayOutput;
	}

	protected XLog copyTraceAttributesToLog(XLog originalLog, Set<String> logAttributesSet) {
		XFactory xFactory = XFactoryRegistry.instance().currentDefault();
		XConceptExtension ce = XConceptExtension.instance();
		XLog modifiedLog = xFactory.createLog(originalLog.getAttributes());
		for (XTrace trace : originalLog) {
			XTrace newTrace = xFactory.createTrace(trace.getAttributes());
			for (XEvent event : trace) {
				XAttributeMap newMap = xFactory.createAttributeMap();
				for (XAttribute attribute : event.getAttributes().values()) {
					XAttribute newAttribute = (XAttribute) attribute.clone();
					newMap.put(newAttribute.getKey(), newAttribute);
					logAttributesSet.add(newAttribute.getKey());
				}
				//copy all trace attributes to events
				for (XAttribute attribute : trace.getAttributes().values()) {
					XAttribute newAttribute;
					if (attribute instanceof XAttributeContinuous)
						newAttribute = new XAttributeContinuousImpl("trace_" + attribute.getKey(),
								((XAttributeContinuous) attribute).getValue());
					else if (attribute instanceof XAttributeDiscrete)
						newAttribute = new XAttributeDiscreteImpl("trace_" + attribute.getKey(),
								((XAttributeDiscrete) attribute).getValue());
					else if (attribute instanceof XAttributeTimestamp)
						newAttribute = new XAttributeTimestampImpl("trace_" + attribute.getKey(),
								((XAttributeTimestamp) attribute).getValue());
					else if (attribute instanceof XAttributeBoolean)
						newAttribute = new XAttributeBooleanImpl("trace_" + attribute.getKey(),
								((XAttributeBoolean) attribute).getValue());
					else
						newAttribute = new XAttributeLiteralImpl("trace_" + attribute.getKey(), attribute.toString());

					newMap.put(newAttribute.getKey(), newAttribute);
					logAttributesSet.add(newAttribute.getKey());
				}
				//				XAttribute nameAttribute = new XAttributeLiteralImpl(NAME_ATTRIBUTE_NAME, ce.extractName(event));
				//				newMap.put(nameAttribute.getKey(), nameAttribute);
				//Add Name attribute

				XEvent newEvent = xFactory.createEvent(newMap);
				newTrace.add(newEvent);
			}
			modifiedLog.add(newTrace);
		}
		return modifiedLog;
	}

	protected XLog prepareLogForDataReplay(XLog log, XEventClassifier classifier, Set<XEventClass> selectedActivities,
			Set<XEventClass> activitiesThatUpdateAttributesSet, Entry<String, String> conditionAttribute) {
		XConceptExtension ce = XConceptExtension.instance();
		XLogInfo logInfo = XLogInfoImpl.create(log, classifier);
		for (XTrace trace : log) {
			for (XEvent event : trace) {
				XAttributeLiteral dataConditionAttribute;
				XEventClass evClass = logInfo.getEventClasses().getClassOf(event);
				if (event.getAttributes().get(conditionAttribute.getValue()) != null
						&& ((!selectedActivities.contains(evClass) && activitiesThatUpdateAttributesSet
								.contains(DataComplianceHelper.getOmegaClass(classifier))) || activitiesThatUpdateAttributesSet
								.contains(evClass))) {
					dataConditionAttribute = new XAttributeLiteralImpl(DATA_CONDITION_ATTR_NAME,
							conditionAttribute.getValue() + "Write");
				} else {
					dataConditionAttribute = new XAttributeLiteralImpl(DATA_CONDITION_ATTR_NAME,
							DataComplianceHelper.OMEGA_CLASS_NAME);
					event.getAttributes().remove(conditionAttribute.getValue());
				}
				event.getAttributes().put(dataConditionAttribute.getKey(), dataConditionAttribute);

				if (!selectedActivities.contains(evClass)) {
					XAttributeLiteral nameAttribute = new XAttributeLiteralImpl(NAME_ATTRIBUTE_NAME,
							DataComplianceHelper.OMEGA_CLASS_NAME);
					event.getAttributes().put(NAME_ATTRIBUTE_NAME, nameAttribute);
					//					XAttributeLiteral nameAttribute = (XAttributeLiteral) event.getAttributes()
					//							.get(NAME_ATTRIBUTE_NAME);
					//					nameAttribute.setValue(DataComplianceHelper.OMEGA_CLASS_NAME);
				} else {
					XAttributeLiteral nameAttribute = new XAttributeLiteralImpl(NAME_ATTRIBUTE_NAME, logInfo
							.getEventClasses().getClassOf(event).toString());
					event.getAttributes().put(NAME_ATTRIBUTE_NAME, nameAttribute);
				}

				ce.assignName(event, dataConditionAttribute.getValue());
			}
		}

		return log;
	}

	protected XLog prepareLogForCFReplay(XLog modifiedLog, DashboardResultReplay dataRepRes,
			Entry<String, String> conditionAttribute) {
		XFactory xFactory = XFactoryRegistry.instance().currentDefault();
		XLog result = xFactory.createLog(modifiedLog.getAttributes());
		XConceptExtension ce = XConceptExtension.instance();
		XLogInfo logInfo = XLogInfoImpl.create(modifiedLog);
		for (DashboardAlignment alignment : dataRepRes.labelStepArray) {
			XTrace trace = DataComplianceHelper.findTraceByName(alignment.getTraceName(), modifiedLog);
			result.add(trace);
			for (int i = 0, l = 0; i < alignment.getStepTypes().size(); i++) {
				if (alignment.getStepTypes().get(i) == DataAwareStepTypes.MINVI
						|| alignment.getStepTypes().get(i) == DataAwareStepTypes.MREAL)
					continue;
				ExecutionStep logStep = alignment.getLogTrace().get(i);
				ExecutionStep processStep = alignment.getProcessTrace().get(i);
				XAttributeMap eventAttributes = trace.get(l).getAttributes();
				XAttributeLiteral dataConditionAttribute = (XAttributeLiteral) eventAttributes
						.get(DATA_CONDITION_ATTR_NAME);
				XAttributeLiteral nameAttribute = (XAttributeLiteral) eventAttributes.get(NAME_ATTRIBUTE_NAME);
				XAttributeLiteral conditionHoldsAttribute = new XAttributeLiteralImpl(CONDITION_HOLDS_ATTR_NAME, "");
				XAttributeLiteral combinedNameConditionHoldsAttribute = new XAttributeLiteralImpl(
						NAME_AND_CONDITION_HOLDS_ATTR_NAME, "");

				if (alignment.getStepTypes().get(i) == DataAwareStepTypes.L)
					conditionHoldsAttribute.setValue(CONDITION_HOLDS_IRRELEVANT);
				else if (dataConditionAttribute.getValue().equals(DataComplianceHelper.OMEGA_CLASS_NAME))
					conditionHoldsAttribute.setValue(CONDITION_HOLDS_IRRELEVANT);
				else if (logStep.get(conditionAttribute.getKey()) != null
						&& logStep.get(conditionAttribute.getKey())
								.equals(processStep.get(conditionAttribute.getKey())))
					conditionHoldsAttribute.setValue(CONDITION_HOLDS_TRUE);
				else
					conditionHoldsAttribute.setValue(CONDITION_HOLDS_FALSE);
				combinedNameConditionHoldsAttribute.setValue(nameAttribute.getValue() + "_"
						+ conditionHoldsAttribute.getValue());
				eventAttributes.put(conditionHoldsAttribute.getKey(), conditionHoldsAttribute);
				eventAttributes.put(combinedNameConditionHoldsAttribute.getKey(), combinedNameConditionHoldsAttribute);
				ce.assignName(trace.get(l), combinedNameConditionHoldsAttribute.getValue());

				l++;
			}
		}
		//		result = modifiedLog;
		return result;
	}

}
