package org.processmining.plugins.datacompliance;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.DataElement;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PNWDTransition;
import org.processmining.models.guards.Expression;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.DataConformance.GUI.MappingPanel;
import org.processmining.plugins.compliance.MapEvPattern2Trans_Smart_UI;
import org.processmining.plugins.compliance.align.PNLogReplayer;
import org.processmining.plugins.compliance.align.PNLogReplayer.ReplayParams;
import org.processmining.plugins.compliance.temporal.CreateTemporalPattern_Plugin;
import org.processmining.plugins.complianceframework.DPNReplayer;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;

public class DataComplianceHelper {

	public static final String OMEGA_CLASS_NAME = "other";
	protected static XEventClass OMEGA_CLASS = null;

	public static XEventClass getOmegaClass(XEventClassifier classifier) {
		if (OMEGA_CLASS == null) {
			if (classifier.getDefiningAttributeKeys().length > 1)
				OMEGA_CLASS = new XEventClass(OMEGA_CLASS_NAME + "+", -17);
			else
				OMEGA_CLASS = new XEventClass(OMEGA_CLASS_NAME, -17);
		} else {
			if (classifier.getDefiningAttributeKeys().length > 1 && !OMEGA_CLASS.getId().endsWith("+"))
				OMEGA_CLASS = new XEventClass(OMEGA_CLASS_NAME + "+", -17);
			else if (classifier.getDefiningAttributeKeys().length == 1 && OMEGA_CLASS.getId().endsWith("+"))
				OMEGA_CLASS = new XEventClass(OMEGA_CLASS_NAME, -17);
		}
		return OMEGA_CLASS;
	}

	public static final String OMEGA_TRANSITION_NAME = "Omega";

	public static XLog abstractLog(XLog originalLog, PetrinetGraph petriNet, TransEvClassMapping mapping,
			XEventClassifier selectedClassifier) {
		XLogInfo info = XLogInfoFactory.createLogInfo(originalLog, selectedClassifier);
		Set<XEventClass> mappedEvClassesSet = new HashSet<XEventClass>();
		for (Transition t : petriNet.getTransitions()) {
			XEventClass evClass = mapping.get(t);
			if (evClass != null)
				mappedEvClassesSet.add(evClass);
		}

		//		XConceptExtension ce = XConceptExtension.instance();
		XFactory xFactory = XFactoryRegistry.instance().currentDefault();
		XLog modifiedLog = xFactory.createLog(originalLog.getAttributes());
		for (XTrace trace : originalLog) {
			XTrace newTrace = xFactory.createTrace(trace.getAttributes());
			for (XEvent event : trace) {
				XAttributeMap newMap = xFactory.createAttributeMap();
				for (XAttribute attribute : event.getAttributes().values()) {
					XAttribute newAttribute = (XAttribute) attribute.clone();
					newMap.put(newAttribute.getKey(), newAttribute);
				}
				XEvent newEvent = xFactory.createEvent(newMap);
				if (!mappedEvClassesSet.contains(info.getEventClasses().getClassOf(newEvent)))
					//					ce.assignName(newEvent, OMEGA_CLASS_NAME);
					assignOmegaClass(newEvent, selectedClassifier.getDefiningAttributeKeys());
				newTrace.add(newEvent);
			}
			modifiedLog.add(newTrace);
		}

		for (Transition t : petriNet.getTransitions()) {
			if (t.getLabel().equals(OMEGA_TRANSITION_NAME))
				mapping.put(t, getOmegaClass(selectedClassifier));
		}

		return modifiedLog;
	}

	public static void assignOmegaClass(XEvent event, String[] definingAttributeKeys) {
		for (int i = 0; i < definingAttributeKeys.length; i++) {
			if (i == 0)
				((XAttributeLiteral) event.getAttributes().get(definingAttributeKeys[i])).setValue(OMEGA_CLASS_NAME);
			else
				((XAttributeLiteral) event.getAttributes().get(definingAttributeKeys[i])).setValue("");
		}
	}

	public static Object[] getActivitiesMapping(UIPluginContext context, XLog log, PetrinetGraph petriNet,
			XEventClassifier[] availableEventClass) {
		for (Transition t : petriNet.getTransitions()) {
			if (t.getLabel().toLowerCase().equalsIgnoreCase("tau")
					|| t.getLabel().toLowerCase().startsWith("final")
					|| t.getLabel().toLowerCase().equals(CreateTemporalPattern_Plugin.T_INSTANCE_START.toLowerCase())
					|| t.getLabel().toLowerCase()
							.equals(CreateTemporalPattern_Plugin.T_INSTANCE_COMPLETE.toLowerCase())
					|| t.getLabel().toLowerCase().equals(CreateTemporalPattern_Plugin.T_START.toLowerCase())
					|| t.getLabel().toLowerCase().equals(CreateTemporalPattern_Plugin.T_END.toLowerCase())) {
				t.setInvisible(true);
			}
		}

		MapEvPattern2Trans_Smart_UI ui = new MapEvPattern2Trans_Smart_UI(log, petriNet, availableEventClass);
		InteractionResult result = context.showWizard("Mapping Petrinet - Event Class of Log", true, true, ui);
		if (result != InteractionResult.FINISHED) {
			return null;
		}

		TransEvClassMapping mapping = ui.getEventMap();
		XEventClassifier classifier = ui.getSelectedClassifier();

		return new Object[] { mapping, classifier };
	}

	public static Map<String, String> getVariableMapping(UIPluginContext context, Set<String> logAttributesSet,
			DataPetriNet DPNet) {
		//		context.getProgress().inc();
		TreeSet<String> processAttributes = new TreeSet<String>();
		for (DataElement variable : DPNet.getVariables()) {
			processAttributes.add(variable.getVarName());
		}
		for (int i = 0; i < DPNReplayer.metaAttributes.length; i++) {
			if (logAttributesSet.contains(DPNReplayer.metaAttributes[i])) {
				logAttributesSet.remove(DPNReplayer.metaAttributes[i]);
			}
		}
		
		MappingPanel<String, String> mapVariablePanel = new MappingPanel<String, String>(processAttributes,
				logAttributesSet);
		InteractionResult interactionResult = context.showConfiguration("Setup Variable Mapping", mapVariablePanel);
		if (interactionResult == InteractionResult.CANCEL)
			return null;
		else {
			Map<String, String> variableMapping = mapVariablePanel.getMapping(false);
			return variableMapping;
		}
	}

	public static PNRepResult cfReplayLog(UIPluginContext context, PetrinetGraph net, TransEvClassMapping mapping,
			XLog log) {
		for (Transition t : net.getTransitions()) {
			if (t.getLabel().toLowerCase().equalsIgnoreCase("tau")
					|| t.getLabel().toLowerCase().startsWith("final")
					|| t.getLabel().toLowerCase().equals(CreateTemporalPattern_Plugin.T_INSTANCE_START.toLowerCase())
					|| t.getLabel().toLowerCase()
							.equals(CreateTemporalPattern_Plugin.T_INSTANCE_COMPLETE.toLowerCase())
					|| t.getLabel().toLowerCase().equals(CreateTemporalPattern_Plugin.T_START.toLowerCase())
					|| t.getLabel().toLowerCase().equals(CreateTemporalPattern_Plugin.T_END.toLowerCase())) {
				t.setInvisible(true);
			}
		}

		//		Object[] abstraction = AbstractLogForCompliance_Plugin.connectLogToPetriNet(context, originalLog, net);
		//		if (abstraction == null) {
		//			return null;
		//		}

		Marking m_initial;
		try {
			m_initial = context.tryToFindOrConstructFirstObject(Marking.class, InitialMarkingConnection.class,
					InitialMarkingConnection.MARKING, net);
		} catch (ConnectionCannotBeObtained e) {
			return null;
		}

		Marking m_final = (Marking) PNLogReplayer.constructFinalMarking(context, net)[1];

		ReplayParams par = PNLogReplayer.getReplayerParameters(context, net, log, mapping.getEventClassifier(),
				m_initial, m_final);
		par.mapEvClass2Cost.put(new XEventClass(OMEGA_CLASS_NAME+"+", -17), 100); //a hack to prevent nullpointerexception
		for (XEventClass e: mapping.values()) {
			par.mapEvClass2Cost.put(e, 100);
		}
		for (XEventClass e : par.mapEvClass2Cost.keySet()) {
			par.mapEvClass2Cost.put(e, 100);
		}
		//		par.mapEvClass2Cost.remove("Omega)
		par.parameters.setCreateConn(true);
		PNRepResult res = PNLogReplayer.callReplayer(context, net, log, mapping, par);

		return res;
	}

	public static Map<String, Expression> getGuardsMap(DataPetriNet DPNet) {
		Map<String, Expression> guardsMap = new HashMap<String, Expression>();
		for (Transition t : DPNet.getTransitions()) {
			Expression expression = ((PNWDTransition) t).getGuard();
			if (expression == null)
				continue;
			guardsMap.put(t.getLabel(), expression);
		}

		return guardsMap;
	}

	public static XTrace findTraceByName(String traceName, XLog log) {
		XTrace result = null;
		for (XTrace trace : log) {
			if (XConceptExtension.instance().extractName(trace).equals(traceName)) {
				result = trace;
				break;
			}
		}
		return result;
	}
	
	public static Map<String, Transition> getTransitionsMap(DataPetriNet DPNet) {
		Map<String, Transition> transitionsMap = new HashMap<String, Transition>();
		for (Transition t : DPNet.getTransitions()) {
			transitionsMap.put(t.getLabel(), t);
		}

		return transitionsMap;
	}
}
