package org.processmining.plugins.logmanipulator.GUI;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import org.processmining.framework.util.ui.widgets.ProMTextArea;
import org.processmining.plugins.logmanipulator.model.AbstractFilter;
import org.processmining.plugins.logmanipulator.model.CaseIDFilter;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class CaseIDFilterConfigurationPanel extends JDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected AbstractFilter newFilter;

	protected JRadioButton rbtn_literal;
	protected JRadioButton rbtn_numeric;
	protected JPanel configurationPanel;
	protected ProMTextArea ta_filterText;

	//	protected ActionListener attrSelectedListener = new ActionListener() {
	//
	//		public void actionPerformed(ActionEvent e) {
	//		}
	//	};

	protected ActionListener addButtonListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			CaseIDFilterConfigurationPanel.this.createFilter();
			CaseIDFilterConfigurationPanel.this.setVisible(false);
		}
	};

	protected ActionListener cancelButtonListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			CaseIDFilterConfigurationPanel.this.setVisible(false);
		}
	};

	public AbstractFilter getNewFilter() {
		return this.newFilter;
	}

	public CaseIDFilterConfigurationPanel(Frame owner) {
		super(owner, "Add case ID filter", true);
		this.newFilter = null;
		this.init();
	}

	public CaseIDFilterConfigurationPanel(Frame owner, CaseIDFilter oldFilter) {
		this(owner);
		StringBuilder filterTextBuilder = new StringBuilder();
		if (oldFilter.isLiteral()) {
			this.rbtn_literal.setSelected(true);
			for (String pattern : oldFilter.getPatterns()) {
				filterTextBuilder.append(pattern);
				filterTextBuilder.append(",");
			}
			if (filterTextBuilder.length() > 0)
				filterTextBuilder.setLength(filterTextBuilder.length() - 1);
			this.ta_filterText.setText(filterTextBuilder.toString());
		} else {
			this.rbtn_numeric.setSelected(true);
			for (int[] interval : oldFilter.getIntervals()) {
				if (interval[0] > Integer.MIN_VALUE)
					filterTextBuilder.append(interval[0]);
				filterTextBuilder.append("--");
				if (interval[1] < Integer.MAX_VALUE)
					filterTextBuilder.append(interval[1]);
				filterTextBuilder.append(",");
			}
			if (filterTextBuilder.length() > 0)
				filterTextBuilder.setLength(filterTextBuilder.length() - 1);
			this.ta_filterText.setText(filterTextBuilder.toString());
		}
	}

	public void init() {
		this.configurationPanel = SlickerFactory.instance().createGradientPanel();
		BoxLayout layout = new BoxLayout(this.configurationPanel, BoxLayout.Y_AXIS);
		this.configurationPanel.setLayout(layout);
		this.configurationPanel.setPreferredSize(new Dimension(500, 300));

		JPanel buttonsPanel = SlickerFactory.instance().createGradientPanel();
		BoxLayout buttonsLayout = new BoxLayout(buttonsPanel, BoxLayout.X_AXIS);
		buttonsPanel.setLayout(buttonsLayout);

		this.getContentPane().add(this.configurationPanel, BorderLayout.CENTER);
		this.getContentPane().add(buttonsPanel, BorderLayout.SOUTH);

		this.rbtn_literal = SlickerFactory.instance().createRadioButton("Consider IDs as literals");
		this.rbtn_literal.setAlignmentX(CENTER_ALIGNMENT);
		this.rbtn_literal.setSelected(true);
		this.rbtn_numeric = SlickerFactory.instance().createRadioButton("Consider IDs as numbers");
		this.rbtn_numeric.setAlignmentX(CENTER_ALIGNMENT);

		ButtonGroup group = new ButtonGroup();
		group.add(rbtn_literal);
		group.add(rbtn_numeric);

		this.ta_filterText = new ProMTextArea(true);
		this.ta_filterText.setAlignmentX(CENTER_ALIGNMENT);

		this.configurationPanel.add(this.rbtn_literal);
		this.configurationPanel.add(this.rbtn_numeric);
		JLabel l = SlickerFactory.instance().createLabel("Specify patterns or intervals separated by ','");
		l.setAlignmentX(CENTER_ALIGNMENT);
		this.configurationPanel.add(l);
		this.configurationPanel.add(this.ta_filterText);

		JButton btn_cancel = SlickerFactory.instance().createButton("Cancel");
		btn_cancel.addActionListener(this.cancelButtonListener);
		JButton btn_add = SlickerFactory.instance().createButton("Add filter");
		btn_add.addActionListener(this.addButtonListener);

		buttonsPanel.add(Box.createHorizontalStrut(10));
		buttonsPanel.add(btn_cancel);
		buttonsPanel.add(Box.createHorizontalGlue());
		buttonsPanel.add(btn_add);
		buttonsPanel.add(Box.createHorizontalStrut(10));

		this.pack();
	}

	protected void createFilter() {

		if (this.rbtn_literal.isSelected()) {
			String[] allowedAr = this.ta_filterText.getText().split(",");
			List<Object> allowedLiteralValues = new ArrayList<Object>();
			for (int i = 0; i < allowedAr.length; i++) {
				allowedLiteralValues.add(allowedAr[i]);
			}
			this.newFilter = new CaseIDFilter(true, allowedLiteralValues);
		} else if (this.rbtn_numeric.isSelected()) {
			String[] intervalStrings = this.ta_filterText.getText().split(",");
			List<Object> allowedDiscreteValues = new ArrayList<Object>();
			for (int i = 0; i < intervalStrings.length; i++) {
				int[] interval = new int[2];
				interval[0] = Integer.MIN_VALUE;
				interval[1] = Integer.MAX_VALUE;
				String[] intervalString = intervalStrings[i].split("--");
				if (intervalString.length == 1) {
					if (intervalStrings[i].startsWith("--"))
						interval[1] = Integer.parseInt(intervalString[0]);
					else
						interval[0] = Integer.parseInt(intervalString[0]);
				} else {
					interval[0] = Integer.parseInt(intervalString[0]);
					interval[1] = Integer.parseInt(intervalString[1]);
				}
				allowedDiscreteValues.add(interval);
			}
			this.newFilter = new CaseIDFilter(false, allowedDiscreteValues);
		}

		//		AttributeFilter newFilter = new AttributeFilter(, , type)
	}
}
