package org.processmining.plugins.logmanipulator.GUI;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedHashMap;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.util.ui.widgets.ProMList;
import org.processmining.plugins.logmanipulator.model.AbstractFilter;
import org.processmining.plugins.logmanipulator.model.AttributeFilter;
import org.processmining.plugins.logmanipulator.model.CaseIDFilter;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class ManipulatorFilterPanel extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected UIPluginContext context;
	protected XLog log;

	protected ProMList<String> filterList;
	protected DefaultListModel filterListModel;
	protected int filterIDCounter;

	protected JButton btn_AddAttrFilter;
	protected JButton btn_AddCaseIDFilter;
	protected JButton btn_EditFilter;
	protected JButton btn_RemoveFilter;

	protected LinkedHashMap<String, AbstractFilter> filtersMap;

	protected ActionListener addAttrFilterListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			Frame owner = JOptionPane.getFrameForComponent(ManipulatorFilterPanel.this);
			AttributeFilterConfigurationPanel attrFilter = new AttributeFilterConfigurationPanel(owner,
					ManipulatorFilterPanel.this.log);
			attrFilter.setLocationRelativeTo(owner);
			attrFilter.setVisible(true);
			if (attrFilter.getNewFilter() != null) {
				ManipulatorFilterPanel.this.addFilter(attrFilter.getNewFilter());
			}
		}
	};

	protected ActionListener addCaseIDFilterListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			Frame owner = JOptionPane.getFrameForComponent(ManipulatorFilterPanel.this);
			CaseIDFilterConfigurationPanel caseIDFilter = new CaseIDFilterConfigurationPanel(owner);
			caseIDFilter.setLocationRelativeTo(owner);
			caseIDFilter.setVisible(true);
			if (caseIDFilter.getNewFilter() != null) {
				ManipulatorFilterPanel.this.addFilter(caseIDFilter.getNewFilter());
			}
		}
	};

	protected ActionListener editFilterListener = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (ManipulatorFilterPanel.this.filterList.getSelectedValuesList().size() == 1) {
				String filterID = ManipulatorFilterPanel.this.filterList.getSelectedValuesList().get(0);
				//				int filterIndex = ManipulatorFilterPanel.this.filterListModel.indexOf(filterID);
				AbstractFilter selectedFilter = ManipulatorFilterPanel.this.filtersMap.get(filterID);
				if (selectedFilter != null) {
					if (selectedFilter instanceof AttributeFilter) {
						Frame owner = JOptionPane.getFrameForComponent(ManipulatorFilterPanel.this);
						AttributeFilterConfigurationPanel attrFilterPanel = new AttributeFilterConfigurationPanel(
								owner, ManipulatorFilterPanel.this.log, (AttributeFilter) selectedFilter);
						attrFilterPanel.setLocationRelativeTo(owner);
						attrFilterPanel.setVisible(true);
						if (attrFilterPanel.getNewFilter() != null) {
							ManipulatorFilterPanel.this.filtersMap.put(filterID, attrFilterPanel.getNewFilter());
						}
					} else if (selectedFilter instanceof CaseIDFilter) {
						Frame owner = JOptionPane.getFrameForComponent(ManipulatorFilterPanel.this);
						CaseIDFilterConfigurationPanel caseIDFilter = new CaseIDFilterConfigurationPanel(owner,
								(CaseIDFilter) selectedFilter);
						caseIDFilter.setLocationRelativeTo(owner);
						caseIDFilter.setVisible(true);
						if (caseIDFilter.getNewFilter() != null) {
							ManipulatorFilterPanel.this.filtersMap.put(filterID, caseIDFilter.getNewFilter());
						}
					}
				}
			}
		}
	};

	protected ActionListener removeFilterListener = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (ManipulatorFilterPanel.this.filterList.getSelectedValuesList().size() > 0) {
				int[] filterIndices = new int[ManipulatorFilterPanel.this.filterList.getSelectedValuesList().size()];
				for (int i = 0; i < filterIndices.length; i++) {
					String filterID = ManipulatorFilterPanel.this.filterList.getSelectedValuesList().get(i);
					ManipulatorFilterPanel.this.filtersMap.remove(filterID);
					filterIndices[i] = ManipulatorFilterPanel.this.filterListModel.indexOf(filterID);
				}
				for (int i = filterIndices.length - 1; i >= 0; i--)
					ManipulatorFilterPanel.this.filterListModel.remove(filterIndices[i]);
			}
		}
	};

	protected ListSelectionListener filterListSelectionListener = new ListSelectionListener() {
		public void valueChanged(ListSelectionEvent e) {
			if (ManipulatorFilterPanel.this.filterList.getSelectedValuesList().size() > 0) {
				ManipulatorFilterPanel.this.btn_RemoveFilter.setEnabled(true);
				if (ManipulatorFilterPanel.this.filterList.getSelectedValuesList().size() == 1)
					ManipulatorFilterPanel.this.btn_EditFilter.setEnabled(true);
				else
					ManipulatorFilterPanel.this.btn_EditFilter.setEnabled(false);
			} else {
				ManipulatorFilterPanel.this.btn_EditFilter.setEnabled(false);
				ManipulatorFilterPanel.this.btn_RemoveFilter.setEnabled(false);
			}
		}
	};

	boolean flag = true;

	public ManipulatorFilterPanel(UIPluginContext context, XLog log) {
		this.context = context;
		this.log = log;
		this.filtersMap = new LinkedHashMap<String, AbstractFilter>();
		this.filterListModel = new DefaultListModel();
		this.filterIDCounter = 0;
		this.init();
	}

	public void init() {
		//		GridLayout layout = new GridLayout(1, 2);
		BorderLayout layout = new BorderLayout(2, 2);
		this.setLayout(layout);

		//configure filter list
		this.filterList = new ProMList<String>("Filter List", filterListModel);
		Dimension filterListSize = new Dimension(5, this.getHeight() - 10);
		this.filterList.setPreferredSize(filterListSize);
		this.filterList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		this.filterList.addListSelectionListener(this.filterListSelectionListener);

		this.add(this.filterList, BorderLayout.CENTER);
		this.add(this.createFilterControlPanel(), BorderLayout.EAST);

	}

	@Override
	public void paint(java.awt.Graphics g) {
		super.paint(g);

		//		if (flag) {
		//			Dimension filterListSize = new Dimension(10, this.getHeight()-10);
		//			this.filterList.setPreferredSize(filterListSize);
		//			this.filterList.setMaximumSize(filterListSize);
		//			flag = false;
		//		}
	}

	protected JPanel createFilterControlPanel() {
		JPanel result = SlickerFactory.instance().createGradientPanel();
		BoxLayout layout = new BoxLayout(result, BoxLayout.Y_AXIS);
		result.setLayout(layout);

		this.btn_AddAttrFilter = SlickerFactory.instance().createButton("Add attribute filter");
		this.btn_AddAttrFilter.setAlignmentX(Component.CENTER_ALIGNMENT);
		this.btn_AddAttrFilter.addActionListener(this.addAttrFilterListener);

		this.btn_AddCaseIDFilter = SlickerFactory.instance().createButton("Add caseID filter");
		this.btn_AddCaseIDFilter.setAlignmentX(Component.CENTER_ALIGNMENT);
		this.btn_AddCaseIDFilter.addActionListener(this.addCaseIDFilterListener);

		this.btn_EditFilter = SlickerFactory.instance().createButton("Edit filter");
		this.btn_EditFilter.setAlignmentX(Component.CENTER_ALIGNMENT);
		this.btn_EditFilter.setEnabled(false);
		this.btn_EditFilter.addActionListener(this.editFilterListener);

		this.btn_RemoveFilter = SlickerFactory.instance().createButton("Remove filter");
		this.btn_RemoveFilter.setAlignmentX(Component.CENTER_ALIGNMENT);
		this.btn_RemoveFilter.setEnabled(false);
		this.btn_RemoveFilter.addActionListener(this.removeFilterListener);

		result.add(Box.createVerticalStrut(10));
		result.add(this.btn_AddAttrFilter);
		result.add(Box.createVerticalStrut(10));
		result.add(this.btn_AddCaseIDFilter);
		result.add(Box.createVerticalStrut(10));
		result.add(this.btn_EditFilter);
		result.add(Box.createVerticalStrut(10));
		result.add(this.btn_RemoveFilter);

		return result;
	}

	protected void addFilter(AbstractFilter filter) {
		String filterID = filter.toString() + " filter " + this.filterIDCounter;
		this.filterIDCounter++;
		this.filtersMap.put(filterID, filter);
		this.filterListModel.addElement(filterID);
	}

	public LinkedHashMap<String, AbstractFilter> getFiltersMap() {
		return this.filtersMap;
	}
}
