package org.processmining.plugins.logmanipulator;

import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.plugins.logmanipulator.GUI.ManipulatorActionsPanel;
import org.processmining.plugins.logmanipulator.GUI.ManipulatorFilterPanel;
import org.processmining.plugins.logmanipulator.model.AbstractFilter;
import org.processmining.plugins.logmanipulator.model.AttributeModifier;

public class LogManipulatorPlugin {
	@Plugin(name = "Manipulate log", parameterLabels = { "Log" }, returnLabels = { "Modified log" }, returnTypes = { XLog.class })
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "V. Gromov", email = "v.gromov@hotmail.com")
	public XLog getDeviationOverview(UIPluginContext context, XLog originalLog) {
		XFactory f = XFactoryRegistry.instance().currentDefault();
		XLog result = f.createLog(originalLog.getAttributes());

		ManipulatorFilterPanel filterStackPanel = new ManipulatorFilterPanel(context, originalLog);

		InteractionResult inResult = context.showConfiguration("Filter events to modify", filterStackPanel);
		if (inResult == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		List<AbstractFilter> filterList = new ArrayList<AbstractFilter>();
		for (Entry<String, AbstractFilter> filterEntry : filterStackPanel.getFiltersMap().entrySet())
			filterList.add(filterEntry.getValue());

		ManipulatorActionsPanel actionStackPanel = new ManipulatorActionsPanel(context, originalLog);

		inResult = context.showConfiguration("Define modifications", actionStackPanel);
		if (inResult == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		List<AttributeModifier> modifierList = new ArrayList<AttributeModifier>();
		for (Entry<String, AttributeModifier> modifierEntry : actionStackPanel.getModfiersMap().entrySet())
			modifierList.add(modifierEntry.getValue());

		XConceptExtension ce = XConceptExtension.instance();
		for (XTrace trace : originalLog) {
			XTrace newTrace = f.createTrace(trace.getAttributes());
			for (XEvent event : trace) {
				XAttributeMap newMap = f.createAttributeMap();
				for (XAttribute attribute: event.getAttributes().values()){
					XAttribute newAttribute = (XAttribute)attribute.clone();
					newMap.put(newAttribute.getKey(), newAttribute);
				}
				XEvent newEvent = f.createEvent(newMap);
				newTrace.add(newEvent);
			}
			if (newTrace.size() > 0)
				result.add(newTrace);
		}

		for (XTrace trace : result)
			for (XEvent event : trace) {
				boolean keep = true;
				for (int i = 0; i < filterList.size(); i++)
					if (!filterList.get(i).keep(event, ce.extractName(trace))) {
						keep = false;
						break;
					}
				if (keep)
					for (int i = 0; i < modifierList.size(); i++)
						modifierList.get(i).modify(event);
			}

		return result;
	}
}
